/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "3DMap.h"
#include "Defs.h"
#include "App.h"

enum {
	Menu_Map_Size_5 = 3421,
	Menu_Map_Size_10,
	Menu_Map_Size_20,
	Menu_Map_Size_30,
	Menu_Map_Size_50,
	Menu_Map_Size_100,
	Menu_Map_Size_200,
	Menu_Map_Size_300
};

BEGIN_EVENT_TABLE(C3DMap, wxPanel)
	EVT_CHAR(C3DMap::onChar)
	EVT_PAINT(C3DMap::onPaint)
	EVT_RIGHT_DOWN(C3DMap::onMouseClick)
	EVT_MENU(Menu_Item_1, C3DMap::onMenuAngle)
	EVT_MENU(Menu_Item_2, C3DMap::onMenuAngle)
	EVT_MENU(Menu_Item_3, C3DMap::onMenuHeight)
	EVT_MENU(Menu_Item_4, C3DMap::onMenuHeight)
	EVT_MENU(Menu_Item_6, C3DMap::onMenuMapType)
	EVT_MENU(Menu_Item_7, C3DMap::onMenuMapType)
	EVT_MENU(Menu_Item_8, C3DMap::onMenuPalette)
	EVT_MENU(Menu_Item_9, C3DMap::onMenuPalette)
	EVT_MENU(Menu_Map_Size_5,   C3DMap::onMenuMapSize)
	EVT_MENU(Menu_Map_Size_10,  C3DMap::onMenuMapSize)
	EVT_MENU(Menu_Map_Size_20,  C3DMap::onMenuMapSize)
	EVT_MENU(Menu_Map_Size_30,  C3DMap::onMenuMapSize)
	EVT_MENU(Menu_Map_Size_50,  C3DMap::onMenuMapSize)
	EVT_MENU(Menu_Map_Size_100, C3DMap::onMenuMapSize)
	EVT_MENU(Menu_Map_Size_200, C3DMap::onMenuMapSize)
	EVT_MENU(Menu_Map_Size_300, C3DMap::onMenuMapSize)
END_EVENT_TABLE()

const int PLOT_SIZE = 200;

C3DMap::C3DMap(wxWindow* parent, const CPoint& point, int mapSize) :
wxPanel(parent, -1, wxDefaultPosition, wxSize(GRAPH_WIDTH, GRAPH_HEIGHT)),
wxTimer(),
m_bitmap(NULL),
m_menu(NULL),
m_menuSize(NULL),
m_map(NULL),
m_maxHeight(-9999),
m_minHeight(9999),
m_point(point),
m_mapSize(mapSize),
m_ulLat(0.0),
m_ulLon(0.0),
m_heightDeg(0.0),
m_widthDeg(0.0),
m_orthAngle(50),
m_mapType(1),
m_xhFactor(0.0),
m_palette(NULL),
m_flash(true),
m_homeX(0),
m_homeY(0)
{
	m_bitmap = new wxBitmap(GRAPH_WIDTH, GRAPH_HEIGHT);
	m_map    = new int[PLOT_SIZE * 2 * PLOT_SIZE * 2];

	m_menuSize = new wxMenu();
	m_menuSize->AppendRadioItem(Menu_Map_Size_5,   wxT("5km"));
	m_menuSize->AppendRadioItem(Menu_Map_Size_10,  wxT("10km"));
	m_menuSize->AppendRadioItem(Menu_Map_Size_20,  wxT("20km"));
	m_menuSize->AppendRadioItem(Menu_Map_Size_30,  wxT("30km"));
	m_menuSize->AppendRadioItem(Menu_Map_Size_50,  wxT("50km"));
	m_menuSize->AppendRadioItem(Menu_Map_Size_100, wxT("100km"));
	m_menuSize->AppendRadioItem(Menu_Map_Size_200, wxT("200km"));
	m_menuSize->AppendRadioItem(Menu_Map_Size_300, wxT("300km"));

	m_menu = new wxMenu();
	m_menu->Append(Menu_Item_1, wxT("Angle -"));
	m_menu->Append(Menu_Item_2, wxT("Angle +"));
	m_menu->AppendSeparator();
	m_menu->Append(Menu_Item_3, wxT("Height -"));
	m_menu->Append(Menu_Item_4, wxT("Height +"));
	m_menu->AppendSeparator();
	m_menu->Append(Menu_Item_5, wxT("Map Size"), m_menuSize);
	m_menu->AppendSeparator();
	m_menu->AppendRadioItem(Menu_Item_6, wxT("Map type 1"));
	m_menu->AppendRadioItem(Menu_Item_7, wxT("Map type 2"));
	m_menu->AppendSeparator();
	m_menu->Append(Menu_Item_8, wxT("Palette -"));
	m_menu->Append(Menu_Item_9, wxT("Palette +"));

	switch (m_mapSize) {
		case 5:
			m_menuSize->Check(Menu_Map_Size_5, true);
			break;
		case 10:
			m_menuSize->Check(Menu_Map_Size_10, true);
			break;
		case 20:
			m_menuSize->Check(Menu_Map_Size_20, true);
			break;
		case 30:
			m_menuSize->Check(Menu_Map_Size_30, true);
			break;
		case 50:
			m_menuSize->Check(Menu_Map_Size_50, true);
			break;
		case 100:
			m_menuSize->Check(Menu_Map_Size_100, true);
			break;
		case 200:
			m_menuSize->Check(Menu_Map_Size_200, true);
			break;
		case 300:
			m_menuSize->Check(Menu_Map_Size_300, true);
			break;
		default:
			wxASSERT(false);
			break;
	}

	m_menu->Check(Menu_Item_7, true);

	m_palette = CPalette::getInstance();
	m_palette->setPaletteNo(0);
	m_palette->prevPalette();

	CLocation location = point.getLocation();

	m_heightDeg = DEG(double(m_mapSize) / R);
	m_widthDeg  = DEG(double(m_mapSize) / R);
//	m_widthDeg  = DEG(double(m_mapSize) / (R * ::cos(RAD(location.getLatitude()))));

	m_ulLat = location.getLatitude()  + m_heightDeg / 2.0;
	m_ulLon = location.getLongitude() - m_widthDeg / 2.0;

	createMapData();

	drawMap();
	drawScale();
	drawInfo();

	Start(1000);
}

C3DMap::~C3DMap()
{
	delete m_bitmap;

	delete[] m_map;
}

void C3DMap::createMapData()
{
	CDEMCache* cache = ::wxGetApp().getDEMCache();

	wxASSERT(cache != NULL);

	for (int y = 0; y < (PLOT_SIZE * 2); y++) {
		double lat = yToLatitude(y);

		for (int x = 0; x < (PLOT_SIZE * 2); x++) {
			double lon = xToLongitude(x);

			int height = cache->getHeight(lat, lon, true);

			if (height == VOID_DATA)
				height = 0;

			m_map[y * PLOT_SIZE * 2 + x] = height;

			m_maxHeight = ::max(m_maxHeight, height);
			m_minHeight = ::min(m_minHeight, height);
		}
	}

	if (m_maxHeight == 0)
		m_maxHeight = 10;

	m_xhFactor = 50.0 / double(m_maxHeight);
}

void C3DMap::drawMap()
{
	int numColours = m_palette->getColourCount();
	double scale1 = 90.0 / double(PLOT_SIZE);
	double scale2 = 180.0 / double(PLOT_SIZE);
	double sc = scale1 * ::sin(RAD(double(m_orthAngle)));
	double cc = scale2 * ::cos(RAD(double(m_orthAngle)));	
	double colScale = double(numColours - 1) / double(m_maxHeight - m_minHeight);

	// Flood the graph area with black to start with
	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);
	dc.BeginDrawing();
	dc.SetBackground(*wxBLACK_BRUSH);
	dc.Clear();

	wxPen** pens = new wxPen*[numColours];
	for (int i = 0; i < numColours; i++)
		pens[i] = new wxPen(m_palette->getColour(i), 1, wxSOLID);

	int lastX = 0, lastY = 0;
	for (int yCount = PLOT_SIZE * 2 - 1; yCount >= 0; yCount--) {
		for (int xCount = 0; xCount < (PLOT_SIZE * 2); xCount++) {
			int height = m_map[yCount * PLOT_SIZE * 2 + xCount];

			double xhHeight = double(height) * m_xhFactor;

			int x = 0, y = 0;
			switch (m_mapType) {
				case 0:
					x = int(double(GRAPH_WIDTH / 2) + double(xCount) * cc - double(yCount) * cc + 0.5);
					y = int(double(GRAPH_HEIGHT - 100) - double(xCount) * sc - double(yCount) * sc - xhHeight + 0.5);
					break;
				case 1:
					x = int(double(GRAPH_WIDTH / 8) + double(xCount) * scale2 + sc * double(yCount) + 0.5);
					y = int(double(GRAPH_HEIGHT - 100) - double(yCount) * cc - xhHeight + 0.5);
					break;
				default:
					wxASSERT(false);
					break;
			}

			if (xCount == PLOT_SIZE && yCount == PLOT_SIZE) {
				m_homeX = x;
				m_homeY = y;
			}

			int col = int(double(height - m_minHeight) * colScale);

			if (col == 0 && height > 0)
				col = 1;

			wxASSERT(col >= 0 && col < numColours);

			if (xCount > 0) {
				dc.SetPen(*pens[col]);
				dc.DrawLine(lastX, lastY, x, y);
				lastX = x;
				lastY = y;
			} else {
				dc.SetPen(*pens[col]);
				dc.DrawPoint(x, y);
				lastX = x;
				lastY = y;
			}
		}
	}

	for (int i = 0; i < numColours; i++)
		delete pens[i];
	delete[] pens;

	dc.EndDrawing();
}

void C3DMap::drawScale()
{
	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);
	dc.BeginDrawing();

	dc.SetTextForeground(*wxWHITE);

	int numColours  = m_palette->getColourCount();
	double colScale = double(numColours - 1) / double(m_maxHeight - m_minHeight);

	for (int i = 0; i < numColours; i++) {
		wxString text;
		text.Printf(wxT("%4dm"), m_minHeight + int(double(i) / colScale));
		dc.DrawText(text, GRAPH_WIDTH - 80, 220 - i * 15);

		wxBrush brush(m_palette->getColour(i), wxSOLID);
		dc.SetBrush(brush);
		dc.DrawRectangle(GRAPH_WIDTH - 30, 220 - i * 15, 15, 15);
	}

	dc.EndDrawing();	
}

void C3DMap::drawInfo()
{
	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);
	dc.BeginDrawing();

	dc.SetTextForeground(*wxWHITE);

	wxString text;
	text.Printf(wxT("%dkm square"), m_mapSize);
	dc.DrawText(text, 20, GRAPH_HEIGHT - 40);

	EFormat format = m_point.getLocation().getFormat();
	wxString loc   = m_point.getLocation().getLocation(format);

	text.Printf(wxT("Centre: %s"), loc.c_str());
	dc.DrawText(text, 20, GRAPH_HEIGHT - 25);

	dc.EndDrawing();	
}

void C3DMap::Notify()
{
	wxClientDC dc(this);
	dc.BeginDrawing();

	dc.SetPen((m_flash) ? *wxWHITE_PEN : *wxBLACK_PEN);
	dc.DrawCircle(m_homeX, m_homeY, 2);

	dc.EndDrawing();	

	m_flash = !m_flash;
}

void C3DMap::onPaint(const wxPaintEvent& event)
{
	wxPaintDC dc(this);

	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

void C3DMap::onChar(wxKeyEvent& event)
{
	int code = event.GetKeyCode();

	wxCommandEvent dummy;

	switch (code) {
		case 'a':
			dummy.SetId(Menu_Item_1);
			onMenuAngle(dummy);
			break;
		case 'A':
			dummy.SetId(Menu_Item_2);
			onMenuAngle(dummy);
			break;
		case 'h':
			dummy.SetId(Menu_Item_3);
			onMenuHeight(dummy);
			break;
		case 'H':
			dummy.SetId(Menu_Item_4);
			onMenuHeight(dummy);
			break;
		case '1':
			dummy.SetId(Menu_Item_6);
			onMenuMapType(dummy);
			break;
		case '2':
			dummy.SetId(Menu_Item_7);
			onMenuMapType(dummy);
			break;
		case 'p':
			dummy.SetId(Menu_Item_8);
			onMenuPalette(dummy);
			break;
		case 'P':
			dummy.SetId(Menu_Item_9);
			onMenuPalette(dummy);
			break;
		case '-':
			switch (m_mapSize) {
				case 300:
					dummy.SetId(Menu_Map_Size_200);
					break;
				case 200:
					dummy.SetId(Menu_Map_Size_100);
					break;
				case 100:
					dummy.SetId(Menu_Map_Size_50);
					break;
				case 50:
					dummy.SetId(Menu_Map_Size_30);
					break;
				case 30:
					dummy.SetId(Menu_Map_Size_20);
					break;
				case 20:
					dummy.SetId(Menu_Map_Size_10);
					break;
				case 10:
					dummy.SetId(Menu_Map_Size_5);
					break;
				case 5:
					dummy.SetId(Menu_Map_Size_300);
					break;
				default:
					wxASSERT(false);
					break;
			}
			onMenuMapSize(dummy);
			break;
		case '+':
			switch (m_mapSize) {
				case 300:
					dummy.SetId(Menu_Map_Size_5);
					break;
				case 200:
					dummy.SetId(Menu_Map_Size_300);
					break;
				case 100:
					dummy.SetId(Menu_Map_Size_200);
					break;
				case 50:
					dummy.SetId(Menu_Map_Size_100);
					break;
				case 30:
					dummy.SetId(Menu_Map_Size_50);
					break;
				case 20:
					dummy.SetId(Menu_Map_Size_30);
					break;
				case 10:
					dummy.SetId(Menu_Map_Size_20);
					break;
				case 5:
					dummy.SetId(Menu_Map_Size_10);
					break;
				default:
					wxASSERT(false);
					break;
			}
			onMenuMapSize(dummy);
			break;
		default:
			event.Skip();
			break;
	}
}

void C3DMap::onMouseClick(const wxMouseEvent& event)
{
	wxPoint point = event.GetPosition();

	PopupMenu(m_menu, point);
}

void C3DMap::onMenuAngle(const wxCommandEvent& event)
{
	switch (event.GetId()) {
		case Menu_Item_1:
			m_orthAngle = (m_orthAngle - 10) % 90;
			break;
		case Menu_Item_2:
			m_orthAngle = (m_orthAngle + 10) % 90;
			break;
		default:
			break;
	}

	drawMap();
	drawScale();
	drawInfo();

	wxClientDC dc(this);
	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

void C3DMap::onMenuHeight(const wxCommandEvent& event)
{
	switch (event.GetId()) {
		case Menu_Item_3:
			m_xhFactor /= 1.2;
			break;
		case Menu_Item_4:
			m_xhFactor *= 1.2;
			break;
		default:
			break;
	}

	drawMap();
	drawScale();
	drawInfo();

	wxClientDC dc(this);
	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

void C3DMap::onMenuMapType(const wxCommandEvent& event)
{
	wxASSERT(m_menu != NULL);

	switch (event.GetId()) {
		case Menu_Item_6:
			m_mapType   = 0;
			m_orthAngle = 50;
			break;
		case Menu_Item_7:
			m_mapType   = 1;
			m_orthAngle = 50;
			break;
		default:
			break;
	}

	m_menu->Check(event.GetId(), true);

	drawMap();
	drawScale();
	drawInfo();

	wxClientDC dc(this);
	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

void C3DMap::onMenuPalette(const wxCommandEvent& event)
{
	switch (event.GetId()) {
		case Menu_Item_8:
			m_palette->prevPalette();
			break;
		case Menu_Item_9:
			m_palette->nextPalette();
			break;
		default:
			break;
	}

	drawMap();
	drawScale();
	drawInfo();

	wxClientDC dc(this);
	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

void C3DMap::onMenuMapSize(const wxCommandEvent& event)
{
	wxASSERT(m_menuSize != NULL);

	switch (event.GetId()) {
		case Menu_Map_Size_5:
			m_mapSize = 5;
			break;
		case Menu_Map_Size_10:
			m_mapSize = 10;
			break;
		case Menu_Map_Size_20:
			m_mapSize = 20;
			break;
		case Menu_Map_Size_30:
			m_mapSize = 30;
			break;
		case Menu_Map_Size_50:
			m_mapSize = 50;
			break;
		case Menu_Map_Size_100:
			m_mapSize = 100;
			break;
		case Menu_Map_Size_200:
			m_mapSize = 200;
			break;
		case Menu_Map_Size_300:
			m_mapSize = 300;
			break;
		default:
			wxASSERT(false);
			break;
	}

	m_menuSize->Check(event.GetId(), true);

	m_maxHeight = -9999;
	m_minHeight = 9999;

	CLocation location = m_point.getLocation();

	m_heightDeg = DEG(double(m_mapSize) / R);
	m_widthDeg  = DEG(double(m_mapSize) / R);
//	m_widthDeg  = DEG(double(m_mapSize) / (R * ::cos(RAD(location.getLatitude()))));

	m_ulLat = location.getLatitude()  + m_heightDeg / 2.0;
	m_ulLon = location.getLongitude() - m_widthDeg / 2.0;

	createMapData();

	drawMap();
	drawScale();
	drawInfo();

	wxClientDC dc(this);
	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

double C3DMap::xToLongitude(int x) const
{
	wxASSERT(x >= 0 || x < (PLOT_SIZE * 2));

	double scale = m_widthDeg / double(PLOT_SIZE * 2);

	return m_ulLon + double(x) * scale;
}

double C3DMap::yToLatitude(int y) const
{
	wxASSERT(y >= 0 || y < (PLOT_SIZE * 2));

	double scale = m_heightDeg / double(PLOT_SIZE * 2);

	return m_ulLat - double(PLOT_SIZE * 2 - y) * scale;
}
