/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "3DMapPreferences.h"
#include "Defs.h"

enum {
	Map_Size_5,
	Map_Size_10,
	Map_Size_20,
	Map_Size_30,
	Map_Size_50,
	Map_Size_100,
	Map_Size_200,
	Map_Size_300
};

BEGIN_EVENT_TABLE(C3DMapPreferences, wxDialog)
	EVT_BUTTON(wxID_OK,     C3DMapPreferences::onOK)
	EVT_BUTTON(wxID_CANCEL, C3DMapPreferences::onCancel)
END_EVENT_TABLE()

C3DMapPreferences::C3DMapPreferences(wxWindow* parent, int mapSize, int id) :
wxDialog(parent, id, wxString(wxT("3DMap Preferences"))),
m_mapSize(NULL)
{
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* entrySizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxHORIZONTAL);

	wxStaticText* heightLabel = new wxStaticText(this, -1, wxT("Map Size"), wxDefaultPosition, wxSize(75, -1));
	entrySizer->Add(heightLabel, 0, wxALL, BORDER_SIZE);

	m_mapSize = new wxChoice(this, -1, wxDefaultPosition, wxSize(100, -1));
	m_mapSize->Append(wxT("5km"),   (void *)Map_Size_5);
	m_mapSize->Append(wxT("10km"),  (void *)Map_Size_10);
	m_mapSize->Append(wxT("20km"),  (void *)Map_Size_20);
	m_mapSize->Append(wxT("30km"),  (void *)Map_Size_30);
	m_mapSize->Append(wxT("50km"),  (void *)Map_Size_50);
	m_mapSize->Append(wxT("100km"), (void *)Map_Size_100);
	m_mapSize->Append(wxT("200km"), (void *)Map_Size_200);
	m_mapSize->Append(wxT("300km"), (void *)Map_Size_300);
	entrySizer->Add(m_mapSize, 0, wxALL, BORDER_SIZE);

	sizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	switch (mapSize) {
		case 5:
			m_mapSize->SetSelection(Map_Size_5);
			break;
		case 10:
			m_mapSize->SetSelection(Map_Size_10);
			break;
		case 20:
			m_mapSize->SetSelection(Map_Size_20);
			break;
		case 30:
			m_mapSize->SetSelection(Map_Size_30);
			break;
		case 50:
			m_mapSize->SetSelection(Map_Size_50);
			break;
		case 100:
			m_mapSize->SetSelection(Map_Size_100);
			break;
		case 200:
			m_mapSize->SetSelection(Map_Size_200);
			break;
		case 300:
			m_mapSize->SetSelection(Map_Size_300);
			break;
		default:
			m_mapSize->SetSelection(Map_Size_50);
			break;
	}

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

C3DMapPreferences::~C3DMapPreferences()
{
}

void C3DMapPreferences::onOK(const wxCommandEvent& event)
{
	wxASSERT(m_mapSize != NULL);

	if (IsModal()) {
		EndModal(wxID_OK);
	} else {
		SetReturnCode(wxID_OK);
		Show(false);
	}
}

void C3DMapPreferences::onCancel(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_CANCEL);
	} else {
		SetReturnCode(wxID_CANCEL);
		Show(false);
	}
}

int C3DMapPreferences::getMapSize() const
{
	wxASSERT(m_mapSize != NULL);

	int mapSize = m_mapSize->GetSelection();

	switch (mapSize) {
		case Map_Size_5:
			return 5;
		case Map_Size_10:
			return 10;
		case Map_Size_20:
			return 20;
		case Map_Size_30:
			return 30;
		case Map_Size_50:
			return 50;
		case Map_Size_100:
			return 100;
		case Map_Size_200:
			return 200;
		case Map_Size_300:
			return 300;
		default:
			wxASSERT(false);
			return 0;
	}
}
