/*
 *   Copyright (C) 2002-2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	App_H
#define	App_H

#include <wx/wx.h>

#include "DEMCache.h"
#include "Location.h"
#include "Point.h"

class CApp : public wxApp {

    public:
	CApp();
	virtual ~CApp();

	virtual bool OnInit();
	virtual int  OnExit();

	virtual void getGeneralPreferences(wxString& srtmPath, wxString& globePath, wxString& gtopo30Path, EDTEM& dtem, wxString& dataPath, wxString& mapQTHPath);
	virtual void get3DMapPreferences(int& mapSize);
	virtual void getCoveragePreferences(int& height, int& frequency, int& txPower, int& txAntGain, int& rxPower, int& rxAntGain, int& polarisation, int& climate, int& ground, int& confidence);
	virtual void getHorizonPreferences(int& distance, double& minElevation, double& maxElevation, double& k);
	virtual void getMapPreferences(EFormat& format, int& resolution, EType& type, bool& towns, bool& sites, int& textSize);

	virtual void setGeneralPreferences(const wxString& srtmPath, const wxString& globePath, const wxString& gtopo30Path, EDTEM dtem, const wxString& dataPath, const wxString& mapQTHPath);
	virtual void set3DMapPreferences(int mapSize);
	virtual void setCoveragePreferences(int height, int frequency, int txPower, int txAntGain, int rxPower, int rxAntGain, int polarisation, int climate, int ground, int confidence);
	virtual void setHorizonPreferences(int distance, double minElevation, double maxElevation, double k);
	virtual void setMapPreferences(EFormat format, int resolution, EType type, bool towns, bool sites, int textSize);

	virtual void getMap(wxString& fileName, CLocation& centre, int& resolution, EType& type, bool& towns, bool& sites);
	virtual void setMap(const wxString& fileName, const CLocation& centre, int resolution, EType type, bool towns, bool sites);

	virtual CDEMCache* getDEMCache();

	virtual CPoint* getPoint(int n);
	virtual int     setPoint(CPoint* location);
	virtual int     countPoints() const;
	virtual bool    removePoint(int n);
	virtual void    clearPoints(double minLat, double minLon, double maxLat, double maxLon);
	virtual void    clearPoints();

    private:
	CDEMCache* m_cache;
	wxString   m_fileName;
	CLocation  m_centre;
	int        m_resolution;
	EType      m_type;
	bool       m_towns;
	bool       m_sites;
	CPoint*    m_points[MAX_POINT_COUNT];
};

DECLARE_APP(CApp)

#endif
