/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "CoveragePreferences.h"

#include "Defs.h"

enum {
	POLARISATION_HORIZONTAL,
	POLARISATION_VERTICAL
};

enum {
	CLIMATE_EQUATORIAL,
	CLIMATE_CONTINENTAL_SUBTROPICAL,
	CLIMATE_MARITIME_TROPICAL,
	CLIMATE_DESERT,
	CLIMATE_CONTINENTAL_TEMPERATE,
	CLIMATE_MARITIME_TEMPERATE_LAND,
	CLIMATE_MARITIME_TEMPERATE_SEA
};

enum {
	GROUND_SALT_WATER,
	GROUND_GOOD,
	GROUND_FRESH_WATER,
	GROUND_MARSHY_LAND,
	GROUND_FARMLAND_FOREST,
	GROUND_AVERAGE,
	GROUND_MOUNTAIN_SAND,
	GROUND_CITY,
	GROUND_POOR
};

BEGIN_EVENT_TABLE(CCoveragePreferences, wxDialog)
	EVT_BUTTON(wxID_OK,     CCoveragePreferences::onOK)
	EVT_BUTTON(wxID_CANCEL, CCoveragePreferences::onCancel)
END_EVENT_TABLE()

CCoveragePreferences::CCoveragePreferences(wxWindow* parent, int height, int frequency, int polarisation, int climate, int ground, int confidence, int txPower, int txAntGain, int rxPower, int rxAntGain, int id) :
wxDialog(parent, id, wxString(wxT("Coverage Preferences"))),
m_height(NULL),
m_frequency(NULL),
m_txPower(NULL),
m_txAntGain(NULL),
m_rxPower(NULL),
m_rxAntGain(NULL),
m_polarisation(NULL),
m_climate(NULL),
m_ground(NULL),
m_confidence(NULL)
{
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* boxSizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxHORIZONTAL);

	wxFlexGridSizer* entrySizer = new wxFlexGridSizer(4);

	wxStaticText* freqLabel = new wxStaticText(this, -1, wxT("Frequency (MHz)"));
	entrySizer->Add(freqLabel, 0, wxALL, BORDER_SIZE);

	m_frequency = new wxTextCtrl(this, -1);
	m_frequency->SetMaxLength(8);
	entrySizer->Add(m_frequency, 0, wxALL, BORDER_SIZE);

	wxStaticText* polLabel = new wxStaticText(this, -1, wxT("Polarisation"));
	entrySizer->Add(polLabel, 0, wxALL, BORDER_SIZE);

	m_polarisation = new wxChoice(this, -1, wxDefaultPosition, wxSize(100, -1));
	m_polarisation->Append(wxT("Horizontal"), (void *)POLARISATION_HORIZONTAL);
	m_polarisation->Append(wxT("Vertical"),   (void *)POLARISATION_VERTICAL);
	entrySizer->Add(m_polarisation, 0, wxALL, BORDER_SIZE);

	wxStaticText* txPwrLabel = new wxStaticText(this, -1, wxT("TX Power (dBm)"));
	entrySizer->Add(txPwrLabel, 0, wxALL, BORDER_SIZE);

	m_txPower = new wxTextCtrl(this, -1);
	m_txPower->SetMaxLength(8);
	entrySizer->Add(m_txPower, 0, wxALL, BORDER_SIZE);

	wxStaticText* txAntLabel = new wxStaticText(this, -1, wxT("TX Antenna Gain (dBi)"));
	entrySizer->Add(txAntLabel, 0, wxALL, BORDER_SIZE);

	m_txAntGain = new wxTextCtrl(this, -1);
	m_txAntGain->SetMaxLength(8);
	entrySizer->Add(m_txAntGain, 0, wxALL, BORDER_SIZE);

	wxStaticText* rxPwrLabel = new wxStaticText(this, -1, wxT("Min. RX Signal (dBm)"));
	entrySizer->Add(rxPwrLabel, 0, wxALL, BORDER_SIZE);

	m_rxPower = new wxTextCtrl(this, -1);
	m_rxPower->SetMaxLength(8);
	entrySizer->Add(m_rxPower, 0, wxALL, BORDER_SIZE);

	wxStaticText* rxAntLabel = new wxStaticText(this, -1, wxT("RX Antenna Gain (dBi)"));
	entrySizer->Add(rxAntLabel, 0, wxALL, BORDER_SIZE);

	m_rxAntGain = new wxTextCtrl(this, -1);
	m_rxAntGain->SetMaxLength(8);
	entrySizer->Add(m_rxAntGain, 0, wxALL, BORDER_SIZE);

	wxStaticText* heightLabel = new wxStaticText(this, -1, wxT("Height (m AGL)"));
	entrySizer->Add(heightLabel, 0, wxALL, BORDER_SIZE);

	m_height = new wxTextCtrl(this, -1);
	m_height->SetMaxLength(8);
	entrySizer->Add(m_height, 0, wxALL, BORDER_SIZE);

	wxStaticText* grndLabel = new wxStaticText(this, -1, wxT("Ground"));
	entrySizer->Add(grndLabel, 0, wxALL, BORDER_SIZE);

	m_ground = new wxChoice(this, -1, wxDefaultPosition, wxSize(100, -1));
	m_ground->Append(wxT("Salt Water"),       (void *)GROUND_SALT_WATER);
	m_ground->Append(wxT("Good Ground"),      (void *)GROUND_GOOD);
	m_ground->Append(wxT("Fresh Water"),      (void *)GROUND_FRESH_WATER);
	m_ground->Append(wxT("Marshy Land"),      (void *)GROUND_MARSHY_LAND);
	m_ground->Append(wxT("Farmland, Forest"), (void *)GROUND_FARMLAND_FOREST);
	m_ground->Append(wxT("Average Ground"),   (void *)GROUND_AVERAGE);
	m_ground->Append(wxT("Mountain, Sand"),   (void *)GROUND_MOUNTAIN_SAND);
	m_ground->Append(wxT("City"),             (void *)GROUND_CITY);
	m_ground->Append(wxT("Poor Ground"),      (void *)GROUND_POOR);
	entrySizer->Add(m_ground, 0, wxALL, BORDER_SIZE);

	wxStaticText* confLabel = new wxStaticText(this, -1, wxT("Confidence (%age)"));
	entrySizer->Add(confLabel, 0, wxALL, BORDER_SIZE);

	m_confidence = new wxSpinCtrl(this, -1);
	m_confidence->SetRange(1, 99);
	m_confidence->SetValue(confidence);
	entrySizer->Add(m_confidence, 0, wxALL, BORDER_SIZE);

	wxStaticText* climLabel = new wxStaticText(this, -1, wxT("Climate"));
	entrySizer->Add(climLabel, 0, wxALL, BORDER_SIZE);

	m_climate = new wxChoice(this, -1, wxDefaultPosition, wxSize(100, -1));
	m_climate->Append(wxT("Equatorial"),                   (void *)CLIMATE_EQUATORIAL);
	m_climate->Append(wxT("Continental Sub-Tropical"),     (void *)CLIMATE_CONTINENTAL_SUBTROPICAL);
	m_climate->Append(wxT("Maritime Tropical"),            (void *)CLIMATE_MARITIME_TROPICAL);
	m_climate->Append(wxT("Desert"),                       (void *)CLIMATE_DESERT);
	m_climate->Append(wxT("Continental Temperate"),        (void *)CLIMATE_CONTINENTAL_TEMPERATE);
	m_climate->Append(wxT("Maritime Temperate over Land"), (void *)CLIMATE_MARITIME_TEMPERATE_LAND);
	m_climate->Append(wxT("Maritime Temperate over Sea"),  (void *)CLIMATE_MARITIME_TEMPERATE_SEA);
	entrySizer->Add(m_climate, 0, wxALL, BORDER_SIZE);

	boxSizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(boxSizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	wxString text;
	text.Printf(wxT("%d"), height);
	m_height->SetValue(text);

	text.Printf(wxT("%d"), frequency);
	m_frequency->SetValue(text);

	text.Printf(wxT("%d"), txPower);
	m_txPower->SetValue(text);

	text.Printf(wxT("%d"), txAntGain);
	m_txAntGain->SetValue(text);

	text.Printf(wxT("%d"), rxPower);
	m_rxPower->SetValue(text);

	text.Printf(wxT("%d"), rxAntGain);
	m_rxAntGain->SetValue(text);

	if (polarisation == 0)
		m_polarisation->SetSelection(POLARISATION_HORIZONTAL);
	else
		m_polarisation->SetSelection(POLARISATION_VERTICAL);

	switch (climate) {
		case 0:
			m_climate->SetSelection(CLIMATE_EQUATORIAL);
			break;
		case 1:
			m_climate->SetSelection(CLIMATE_CONTINENTAL_SUBTROPICAL);
			break;
		case 2:
			m_climate->SetSelection(CLIMATE_MARITIME_TROPICAL);
			break;
		case 3:
			m_climate->SetSelection(CLIMATE_DESERT);
			break;
		case 4:
			m_climate->SetSelection(CLIMATE_CONTINENTAL_TEMPERATE);
			break;
		case 5:
			m_climate->SetSelection(CLIMATE_MARITIME_TEMPERATE_LAND);
			break;
		case 6:
			m_climate->SetSelection(CLIMATE_MARITIME_TEMPERATE_SEA);
			break;
		default:
			m_climate->SetSelection(CLIMATE_MARITIME_TEMPERATE_LAND);
			break;
	}

	switch (ground) {
		case 0:
			m_ground->SetSelection(GROUND_SALT_WATER);
			break;
		case 1:
			m_ground->SetSelection(GROUND_GOOD);
			break;
		case 2:
			m_ground->SetSelection(GROUND_FRESH_WATER);
			break;
		case 3:
			m_ground->SetSelection(GROUND_MARSHY_LAND);
			break;
		case 4:
			m_ground->SetSelection(GROUND_FARMLAND_FOREST);
			break;
		case 5:
			m_ground->SetSelection(GROUND_AVERAGE);
			break;
		case 6:
			m_ground->SetSelection(GROUND_MOUNTAIN_SAND);
			break;
		case 7:
			m_ground->SetSelection(GROUND_CITY);
			break;
		case 8:
			m_ground->SetSelection(GROUND_POOR);
			break;
		default:
			m_ground->SetSelection(GROUND_AVERAGE);
			break;
	}

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

CCoveragePreferences::~CCoveragePreferences()
{
}

void CCoveragePreferences::onOK(const wxCommandEvent& event)
{
	wxASSERT(m_height != NULL);
	wxASSERT(m_frequency != NULL);
	wxASSERT(m_txPower != NULL);
	wxASSERT(m_txAntGain != NULL);
	wxASSERT(m_rxPower != NULL);
	wxASSERT(m_rxAntGain != NULL);
	wxASSERT(m_polarisation != NULL);
	wxASSERT(m_climate != NULL);
	wxASSERT(m_ground != NULL);
	wxASSERT(m_confidence != NULL);

	wxString text = m_height->GetValue();

	if (!text.IsEmpty()) {
		long height;
		text.ToLong(&height);

		if (height < 0L) {
			wxMessageDialog dialog(this, wxT("The height must be greater then zero"), wxT("UKW Tools Error"), wxICON_ERROR);
			dialog.ShowModal();
			return;
		}
	}

	text = m_frequency->GetValue();

	long frequency;
	text.ToLong(&frequency);

	if (frequency < 20L || frequency > 20000L) {
		wxMessageDialog dialog(this, wxT("The frequency must be between 20 and 20000 MHz"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	if (IsModal()) {
		EndModal(wxID_OK);
	} else {
		SetReturnCode(wxID_OK);
		Show(false);
	}
}

void CCoveragePreferences::onCancel(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_CANCEL);
	} else {
		SetReturnCode(wxID_CANCEL);
		Show(false);
	}
}

int CCoveragePreferences::getHeight() const
{
	wxASSERT(m_height != NULL);

	wxString text = m_height->GetValue();

	if (text.IsEmpty())
		return 0;

	long height;
	text.ToLong(&height);

	return int(height);
}

int CCoveragePreferences::getFrequency() const
{
	wxASSERT(m_frequency != NULL);

	wxString text = m_frequency->GetValue();

	long frequency;
	text.ToLong(&frequency);

	return int(frequency);
}

int CCoveragePreferences::getPolarisation() const
{
	wxASSERT(m_polarisation != NULL);

	int polarisation = m_polarisation->GetSelection();

	switch (polarisation) {
		case POLARISATION_HORIZONTAL:
			return 0;
		case POLARISATION_VERTICAL:
			return 1;
		default:
			wxASSERT(false);
			return 0;
	}
}

int CCoveragePreferences::getClimate() const
{
	wxASSERT(m_climate != NULL);

	int climate = m_climate->GetSelection();

	switch (climate) {
		case CLIMATE_EQUATORIAL:
			return 0;
		case CLIMATE_CONTINENTAL_SUBTROPICAL:
			return 1;
		case CLIMATE_MARITIME_TROPICAL:
			return 2;
		case CLIMATE_DESERT:
			return 3;
		case CLIMATE_CONTINENTAL_TEMPERATE:
			return 4;
		case CLIMATE_MARITIME_TEMPERATE_LAND:
			return 5;
		case CLIMATE_MARITIME_TEMPERATE_SEA:
			return 6;
		default:
			wxASSERT(false);
			return 0;
	}
}

int CCoveragePreferences::getGround() const
{
	wxASSERT(m_ground != NULL);

	int ground = m_ground->GetSelection();

	switch (ground) {
		case GROUND_SALT_WATER:
			return 0;
		case GROUND_GOOD:
			return 1;
		case GROUND_FRESH_WATER:
			return 2;
		case GROUND_MARSHY_LAND:
			return 3;
		case GROUND_FARMLAND_FOREST:
			return 4;
		case GROUND_AVERAGE:
			return 5;
		case GROUND_MOUNTAIN_SAND:
			return 6;
		case GROUND_CITY:
			return 7;
		case GROUND_POOR:
			return 8;
		default:
			wxASSERT(false);
			return 0;
	}
}

int CCoveragePreferences::getConfidence() const
{
	wxASSERT(m_confidence != NULL);

	return m_confidence->GetValue();
}

int CCoveragePreferences::getTXPower() const
{
	wxASSERT(m_txPower != NULL);

	wxString text = m_txPower->GetValue();

	long txPower;
	text.ToLong(&txPower);

	return int(txPower);
}

int CCoveragePreferences::getTXAntGain() const
{
	wxASSERT(m_txAntGain != NULL);

	wxString text = m_txAntGain->GetValue();

	long txAntGain;
	text.ToLong(&txAntGain);

	return int(txAntGain);
}

int CCoveragePreferences::getRXPower() const
{
	wxASSERT(m_rxPower != NULL);

	wxString text = m_rxPower->GetValue();

	long rxPower;
	text.ToLong(&rxPower);

	return int(rxPower);
}

int CCoveragePreferences::getRXAntGain() const
{
	wxASSERT(m_rxAntGain != NULL);

	wxString text = m_rxAntGain->GetValue();

	long rxAntGain;
	text.ToLong(&rxAntGain);

	return int(rxAntGain);
}
