/*
 *   Copyright (C) 2002-2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Frame.h"
#include "App.h"
#include "Defs.h"

#include "MapFile.h"
#include "3DMapPreferences.h"
#include "CoveragePreferences.h"
#include "HorizonPreferences.h"
#include "MapPreferences.h"
#include "PathPreferences.h"
#include "MapDialog.h"
#include "Map.h"

enum {
	Menu_File_New         = 100,
	Menu_File_Open,
	Menu_File_Save,
	Menu_File_Saveas,
	Menu_File_Quit,
	Menu_Edit_Preferences = 200,
	Menu_Pref_3DMap       = 300,
	Menu_Pref_Coverage,
	Menu_Pref_Horizon,
	Menu_Pref_Map,
	Menu_Pref_Paths,
	Menu_Help_About       = 400
};

BEGIN_EVENT_TABLE(CFrame, wxFrame)
	EVT_MENU(Menu_File_New, CFrame::onNew)
	EVT_MENU(Menu_File_Open, CFrame::onOpen)
	EVT_MENU(Menu_File_Save, CFrame::onSave)
	EVT_MENU(Menu_File_Saveas, CFrame::onSaveas)
	EVT_MENU(Menu_File_Quit, CFrame::onQuit)
	EVT_MENU(Menu_Pref_3DMap, CFrame::on3DMap)
	EVT_MENU(Menu_Pref_Coverage, CFrame::onCoverage)
	EVT_MENU(Menu_Pref_Horizon, CFrame::onHorizon)
	EVT_MENU(Menu_Pref_Map, CFrame::onMap)
	EVT_MENU(Menu_Pref_Paths, CFrame::onPaths)
	EVT_MENU(Menu_Help_About, CFrame::onAbout)
END_EVENT_TABLE()

CFrame::CFrame() :
wxFrame(NULL, -1, PROGRAM_TITLE),
m_sizer(NULL),
m_graph(NULL)
{
	wxMenu* fileMenu = new wxMenu();
	fileMenu->Append(Menu_File_New,    wxT("&New Map...\tAlt-N"));
	fileMenu->AppendSeparator();
	fileMenu->Append(Menu_File_Open,   wxT("&Open Map\tAlt-O"));
	fileMenu->Append(Menu_File_Save,   wxT("&Save Map\tAlt-S"));
	fileMenu->Append(Menu_File_Saveas, wxT("S&ave Map As...\tAlt-A"));
	fileMenu->AppendSeparator();
	fileMenu->Append(Menu_File_Quit, wxT("E&xit\tAlt-X"));

	wxMenu* prefMenu = new wxMenu();
	prefMenu->Append(Menu_Pref_3DMap,    wxT("&3D Map...\tAlt-3"));
	prefMenu->Append(Menu_Pref_Coverage, wxT("&Coverage...\tAlt-C"));
	prefMenu->Append(Menu_Pref_Horizon,  wxT("&Horizon...\tAlt-H"));
	prefMenu->Append(Menu_Pref_Map,      wxT("&Map...\tAlt-M"));
	prefMenu->Append(Menu_Pref_Paths,    wxT("&Paths...\tAlt-P"));

	wxMenu* editMenu = new wxMenu();
	editMenu->Append(Menu_Edit_Preferences, wxT("&Preferences\tAlt-P"), prefMenu);

	wxMenu* helpMenu = new wxMenu();
	helpMenu->Append(Menu_Help_About, wxT("&About UKW Tools\tF1"));

	wxMenuBar* menuBar = new wxMenuBar();
	menuBar->Append(fileMenu, wxT("&File"));
	menuBar->Append(editMenu, wxT("&Edit"));
	menuBar->Append(helpMenu, wxT("&Help"));

	SetMenuBar(menuBar);

	m_sizer = new wxBoxSizer(wxVERTICAL);

	m_graph = new wxWindow(this, -1, wxDefaultPosition, wxSize(WINDOW_WIDTH, WINDOW_HEIGHT));

	m_sizer->Add(m_graph, 0);

	SetAutoLayout(true);

	m_sizer->Fit(this);
	m_sizer->SetSizeHints(this);
	SetSizer(m_sizer);
}

CFrame::~CFrame()
{
}

void CFrame::onNew(const wxCommandEvent& event)
{
	EFormat format;
	EType type;
	int resolution, textSize;
	bool towns, sites;

	::wxGetApp().getMapPreferences(format, resolution, type, towns, sites, textSize);

	CMapDialog dialog(this, format, resolution, type, towns, sites);

	if (dialog.ShowModal() != wxID_OK)
		return;

	CLocation* location = dialog.getLocation();
	resolution          = dialog.getResolution();
	type                = dialog.getType();
	towns               = dialog.getTowns();
	sites               = dialog.getSites();

	::wxGetApp().setMap(wxEmptyString, *location, resolution, type, towns, sites);

	setGraph(new CMap(this, *location, resolution, type, towns, sites, textSize));
}

void CFrame::onOpen(const wxCommandEvent& event)
{
	wxString srtmPath, globePath, gtopo30Path, dataPath, mapQTHPath;
	EDTEM dtem;

	::wxGetApp().getGeneralPreferences(srtmPath, globePath, gtopo30Path, dtem, dataPath, mapQTHPath);

	wxFileDialog dialog(this, wxT("Open Map File"), mapQTHPath, wxEmptyString, wxT("Map Files (*.map;*.MAP)|*.map;*.MAP"));
	dialog.SetStyle(wxOPEN);

	if (dialog.ShowModal() != wxID_OK)
		return;

	wxString fileName = dialog.GetPath();

	CLocation location;
	int resolution, textSize;
	EFormat format;
	EType type;
	bool towns, sites;

	::wxGetApp().getMapPreferences(format, resolution, type, towns, sites, textSize);

	CMapFile file;
	bool ret = file.read(fileName, location, resolution, type, towns, sites);

	if (!ret) {
		wxMessageDialog dialog(this, wxT("Cannot open or parse the Map file"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	::wxGetApp().setMap(fileName, location, resolution, type, towns, sites);

	setGraph(new CMap(this, location, resolution, type, towns, sites, textSize));
}

void CFrame::onSave(const wxCommandEvent& event)
{
	wxString fileName;
	CLocation location;
	int resolution;
	EType type;
	bool towns, sites;

	::wxGetApp().getMap(fileName, location, resolution, type, towns, sites);

	if (fileName.IsEmpty()) {
		onSaveas(event);
		return;
	}

	CMapFile file;
	bool ret = file.write(fileName, location, resolution, type, towns, sites);

	if (!ret) {
		wxMessageDialog dialog(this, wxT("Cannot write the Map file"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}
}

void CFrame::onSaveas(const wxCommandEvent& event)
{
	wxString srtmPath, globePath, gtopo30Path, dataPath, mapQTHPath;
	EDTEM dtem;

	::wxGetApp().getGeneralPreferences(srtmPath, globePath, gtopo30Path, dtem, dataPath, mapQTHPath);

	wxString fileName;
	CLocation location;
	int resolution;
	EType type;
	bool towns, sites;

	::wxGetApp().getMap(fileName, location, resolution, type, towns, sites);

	wxFileDialog dialog(this, wxT("Save Map File"), mapQTHPath, wxEmptyString, wxT("MAP Files (*.map;*.MAP)|*.map;*.MAP"));
	dialog.SetStyle(wxSAVE | wxOVERWRITE_PROMPT);

	if (dialog.ShowModal() != wxID_OK)
		return;

	fileName = dialog.GetPath();

	CMapFile file;
	bool ret = file.write(fileName, location, resolution, type, towns, sites);

	if (!ret) {
		wxMessageDialog dialog(this, wxT("Cannot write the Map file"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	::wxGetApp().setMap(fileName, location, resolution, type, towns, sites);
}

void CFrame::onQuit(const wxCommandEvent& event)
{
	Close(true);
}

void CFrame::on3DMap(const wxCommandEvent& event)
{
	int mapSize;
	::wxGetApp().get3DMapPreferences(mapSize);

	C3DMapPreferences dialog(this, mapSize);

	if (dialog.ShowModal() != wxID_OK)
		return;

	mapSize = dialog.getMapSize();

	::wxGetApp().set3DMapPreferences(mapSize);
}

void CFrame::onCoverage(const wxCommandEvent& event)
{
	int height, frequency, txPower, txAntGain, rxPower, rxAntGain, polarisation, climate, ground, confidence;

	::wxGetApp().getCoveragePreferences(height, frequency, txPower, txAntGain, rxPower, rxAntGain, polarisation, climate, ground, confidence);

	CCoveragePreferences dialog(this, height, frequency, polarisation, climate, ground, confidence, txPower, txAntGain, rxPower, rxAntGain);

	if (dialog.ShowModal() != wxID_OK)
		return;

	height       = dialog.getHeight();
	frequency    = dialog.getFrequency();
	txPower      = dialog.getTXPower();
	txAntGain    = dialog.getTXAntGain();
	rxPower      = dialog.getRXPower();
	rxAntGain    = dialog.getRXAntGain();
	polarisation = dialog.getPolarisation();
	climate      = dialog.getClimate();
	ground       = dialog.getGround();
	confidence   = dialog.getConfidence();

	::wxGetApp().setCoveragePreferences(height, frequency, txPower, txAntGain, rxPower, rxAntGain, polarisation, climate, ground, confidence);
}

void CFrame::onHorizon(const wxCommandEvent& event)
{
	int distance;
	double minElevation, maxElevation, k;

	::wxGetApp().getHorizonPreferences(distance, minElevation, maxElevation, k);

	CHorizonPreferences dialog(this, distance, minElevation, maxElevation, k);

	if (dialog.ShowModal() != wxID_OK)
		return;

	distance     = dialog.getDistance();
	minElevation = dialog.getMinimumElevation();
	maxElevation = dialog.getMaximumElevation();
	k            = dialog.getK();

	::wxGetApp().setHorizonPreferences(distance, minElevation, maxElevation, k);
}

void CFrame::onMap(const wxCommandEvent& event)
{
	EFormat format;
	int resolution, textSize;
	EType type;
	bool towns, sites;

	::wxGetApp().getMapPreferences(format, resolution, type, towns, sites, textSize);

	CMapPreferences dialog(this, format, resolution, type, towns, sites, textSize);

	if (dialog.ShowModal() != wxID_OK)
		return;

	format     = dialog.getFormat();
	resolution = dialog.getResolution();
	type       = dialog.getType();
	towns      = dialog.getTowns();
	sites      = dialog.getSites();
	textSize   = dialog.getTextSize();

	::wxGetApp().setMapPreferences(format, resolution, type, towns, sites, textSize);
}

void CFrame::onPaths(const wxCommandEvent& event)
{
	wxString srtmPath, globePath, gtopo30Path, dataPath, mapQTHPath;
	EDTEM dtem;

	::wxGetApp().getGeneralPreferences(srtmPath, globePath, gtopo30Path, dtem, dataPath, mapQTHPath);

	CPathPreferences dialog(this, srtmPath, globePath, gtopo30Path, dtem, dataPath, mapQTHPath);

	if (dialog.ShowModal() != wxID_OK)
		return;

	srtmPath    = dialog.getSRTMPath();
	globePath   = dialog.getGlobePath();
	gtopo30Path = dialog.getGTOPO30Path();
	dtem        = dialog.getDTEM();
	dataPath    = dialog.getDataPath();
	mapQTHPath  = dialog.getMapQTHPath();

	::wxGetApp().setGeneralPreferences(srtmPath, globePath, gtopo30Path, dtem, dataPath, mapQTHPath);
}

void CFrame::onAbout(const wxCommandEvent& event)
{
	::wxMessageBox(PROGRAM_TITLE + wxT("\n\nWritten by Jonathan Naylor G4KLX\n\n") + RELEASE_DATE + wxT("\n\nBased on the MS-DOS GEOG program suite\nwritten by Andy Talbot G4JNT\nand on SPLAT!, a Terrain Analysis tool for Unix/Linux\nwritten by John A. Magliacane KD2BD"), wxT("About UKW Tools"), wxICON_INFORMATION);
}

void CFrame::setGraph(wxWindow* graph)
{
	wxASSERT(graph != NULL);
	wxASSERT(m_sizer != NULL);

	if (m_graph != NULL) {
		m_sizer->Remove(m_graph);
		delete m_graph;
	}

	m_graph = graph;
	m_sizer->Add(m_graph);
}
