/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "GlobeCache.h"
#include "Defs.h"

#include <wx/log.h>

const int CACHE_SIZE = 4;

CGlobeCache::CGlobeCache(const wxString& path, EDTEM version) :
m_path(path),
m_version(version),
m_list(NULL),
m_ptr(0)
{
	m_list = new CGlobe*[CACHE_SIZE];

	for (int i = 0; i < CACHE_SIZE; i++)
		m_list[i] = NULL;
}

CGlobeCache::~CGlobeCache()
{
	for (int i = 0; i < CACHE_SIZE; i++)
		if (m_list[i] != NULL)
			delete m_list[i];

	delete[] m_list;
}

int CGlobeCache::getHeight(double latitude, double longitude, bool interpolate)
{
	if (longitude > 180.0)
		longitude -= 360.0;

	if (longitude < -180.0)
		longitude += 360.0;

	for (int i = 0; i < CACHE_SIZE; i++) {
		if (m_list[i] != NULL) {
			if (m_list[i]->isInData(latitude, longitude))
				return m_list[i]->getHeight(latitude, longitude);
		}
	}

	if (m_list[m_ptr] != NULL)
		delete m_list[m_ptr];

	m_list[m_ptr] = new CGlobe(m_path, m_version, latitude, longitude);

	int height = m_list[m_ptr]->getHeight(latitude, longitude);

	m_ptr++;

	if (m_ptr == CACHE_SIZE)
		m_ptr = 0;

	return height;
}

bool CGlobeCache::dataExists(double latitude, double longitude)
{
	for (int i = 0; i < CACHE_SIZE; i++) {
		if (m_list[i] != NULL) {
			if (m_list[i]->isInData(latitude, longitude))
				return m_list[i]->dataExists();
		}
	}

	if (m_list[m_ptr] != NULL)
		delete m_list[m_ptr];

	m_list[m_ptr] = new CGlobe(m_path, m_version, latitude, longitude);

	bool exists = m_list[m_ptr]->dataExists();

	m_ptr++;

	if (m_ptr == CACHE_SIZE)
		m_ptr = 0;

	return exists;
}
