/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "HorizonPreferences.h"
#include "Defs.h"

enum {
	K_100,
	K_115,
	K_133,
	K_150
};

BEGIN_EVENT_TABLE(CHorizonPreferences, wxDialog)
	EVT_BUTTON(wxID_OK,     CHorizonPreferences::onOK)
	EVT_BUTTON(wxID_CANCEL, CHorizonPreferences::onCancel)
END_EVENT_TABLE()

CHorizonPreferences::CHorizonPreferences(wxWindow* parent, int distance, double minElevation, double maxElevation, double k, int id) :
wxDialog(parent, id, wxString(wxT("Horizon Preferences"))),
m_distance(NULL),
m_minEle(NULL),
m_maxEle(NULL),
m_k(NULL)
{
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* boxSizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxHORIZONTAL);

	wxFlexGridSizer* entrySizer = new wxFlexGridSizer(2);

	wxStaticText* distLabel = new wxStaticText(this, -1, wxT("Max. Distance (km)"));
	entrySizer->Add(distLabel, 0, wxALL, BORDER_SIZE);

	m_distance = new wxTextCtrl(this, -1);
	m_distance->SetMaxLength(8);
	entrySizer->Add(m_distance, 0, wxALL, BORDER_SIZE);

	wxStaticText* minLabel = new wxStaticText(this, -1, wxT("Minimum Angle ()"));
	entrySizer->Add(minLabel, 0, wxALL, BORDER_SIZE);

	m_minEle = new wxTextCtrl(this, -1);
	m_minEle->SetMaxLength(8);
	entrySizer->Add(m_minEle, 0, wxALL, BORDER_SIZE);

	wxStaticText* maxLabel = new wxStaticText(this, -1, wxT("Maximum Angle ()"));
	entrySizer->Add(maxLabel, 0, wxALL, BORDER_SIZE);

	m_maxEle = new wxTextCtrl(this, -1);
	m_maxEle->SetMaxLength(8);
	entrySizer->Add(m_maxEle, 0, wxALL, BORDER_SIZE);

	wxStaticText* kLabel = new wxStaticText(this, -1, wxT("K Value"));
	entrySizer->Add(kLabel, 0, wxALL, BORDER_SIZE);

	m_k = new wxChoice(this, -1);
	m_k->Append(wxT("1.00"), (void *)K_100);
	m_k->Append(wxT("1.15"), (void *)K_115);
	m_k->Append(wxT("1.33"), (void *)K_133);
	m_k->Append(wxT("1.50"), (void *)K_150);
	entrySizer->Add(m_k, 0, wxALL, BORDER_SIZE);

	boxSizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(boxSizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	wxString text;

	text.Printf(wxT("%d"), distance);
	m_distance->SetValue(text);

	text.Printf(wxT("%.1f"), minElevation);
	m_minEle->SetValue(text);

	text.Printf(wxT("%.1f"), maxElevation);
	m_maxEle->SetValue(text);

	if (k == 1.00)
		m_k->SetSelection(K_100);
	else if (k == 1.15)
		m_k->SetSelection(K_115);
	else if (k == 1.33)
		m_k->SetSelection(K_133);
	else if (k == 1.50)
		m_k->SetSelection(K_150);
	else
		m_k->SetSelection(K_133);

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

CHorizonPreferences::~CHorizonPreferences()
{
}

void CHorizonPreferences::onOK(const wxCommandEvent& event)
{
	wxASSERT(m_distance!= NULL);
	wxASSERT(m_minEle != NULL);
	wxASSERT(m_maxEle != NULL);

	long distance;
	m_distance->GetValue().ToLong(&distance);

	if (distance < 0L) {
		wxMessageDialog dialog(this, wxT("The distance must be greater than zero"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	double minEle, maxEle;
	m_minEle->GetValue().ToDouble(&minEle);
	m_maxEle->GetValue().ToDouble(&maxEle);

	if (maxEle <= minEle) {
		wxMessageDialog dialog(this, wxT("The maximum elevation must be greater than the minimum elevation"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	if (IsModal()) {
		EndModal(wxID_OK);
	} else {
		SetReturnCode(wxID_OK);
		Show(false);
	}
}

void CHorizonPreferences::onCancel(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_CANCEL);
	} else {
		SetReturnCode(wxID_CANCEL);
		Show(false);
	}
}

int CHorizonPreferences::getDistance() const
{
	wxASSERT(m_distance != NULL);

	long distance;
	m_distance->GetValue().ToLong(&distance);

	return int(distance);
}

double CHorizonPreferences::getMinimumElevation() const
{
	wxASSERT(m_minEle != NULL);

	double minElevation;
	m_minEle->GetValue().ToDouble(&minElevation);

	return minElevation;
}

double CHorizonPreferences::getMaximumElevation() const
{
	wxASSERT(m_maxEle != NULL);

	double maxElevation;
	m_maxEle->GetValue().ToDouble(&maxElevation);

	return maxElevation;
}

double CHorizonPreferences::getK() const
{
	wxASSERT(m_k != NULL);

	int k = m_k->GetSelection();

	switch (k) {
		case K_100:
			return 1.00;
		case K_115:
			return 1.15;
		case K_133:
			return 1.33;
		case K_150:
			return 1.50;
		default:
			wxASSERT(false);
			return 0.00;
	}
}
