/****************************************************************************
*                                                                           *
* This file was obtained from ftp://flattop.its.bldrdoc.gov/itm/ITMDLL.cpp  *
* on Jan. 10, 2004 and is public domain.  It was modified by J. D. McDonald *
* to remove Microsoft Windows dll-isms and to correct one case where a      *
* compiler found an ambguity in overloaded calls.  It was further modified  *
* by John A. Magliacane to remove unused variables, unneeded #includes,     *
* and to replace pow() statements with explicit multiplications wherever    *
* possible to increase execution speed and improve accuracy.                *
*                                                                           *
****************************************************************************/

// *************************************
// C++ routines for this program are taken from
// a translation of the FORTRAN code written by
// U.S. Department of Commerce NTIA/ITS
// Institute for Telecommunication Sciences
// *****************
// Irregular Terrain Model (ITM) (Longley-Rice)
// *************************************

#ifndef	ITM_H
#define	ITM_H

#include <wx/wx.h>

#include "Defs.h"

const double THIRD = 1.0 / 3.0;

enum EPol {
	POLARISATION_HORIZONTAL,
	POLARISATION_VERTICAL
};

enum ECriteria {
	CRITERIA_RANDOM,
	CRITERIA_CAREFUL,
	CRITERIA_VERY_CAREFUL
};

enum EClimate {
	CLIMATE_EQUATORIAL,
	CLIMATE_CONTINENTAL_SUBTROPICAL,
	CLIMATE_MARITIME_TROPICAL,
	CLIMATE_DESERT,
	CLIMATE_CONTINENTAL_TEMPERATE,
	CLIMATE_MARITIME_TEMPERATE_LAND,
	CLIMATE_MARITIME_TEMPERATE_SEA
};

class CITM {

    public:
	CITM(int zsys, double surfref, int f, int climate, int ground, int polarisation, int confidence);
	virtual ~CITM();

	virtual void calculate(double elev[], int txHeight, int rxHeight, double& dbloss, wxString& strmode);

	virtual int  getErr() const;

    private:
	double          m_aref;
	double          m_dist;
	double          m_hg[2];
	double          m_wn;
	double          m_dh;
	double          m_ens;
	double          m_gme;
	complex<double> m_zgnd;
	double          m_he[2];
	double          m_dl[2];
	double          m_the[2];
	int             m_kwx;
	double          m_sgc;
	int             m_klim;
	double          m_dlsa;
	double          m_dx;
	double          m_ael;
	double          m_ak1;
	double          m_ak2;
	double          m_aed;
	double          m_emd;
	double          m_aes;
	double          m_ems;
	double          m_dls[2];
	double          m_dla;
	double          m_tha;
	double          m_wd1;
	double          m_xd1;
	double          m_afo;
	double          m_qk;
	double          m_aht;
	double          m_xht;
	double          m_ad;
	double          m_rr;
	double          m_etq;
	double          m_h0s;
	double          m_wls;
	double          m_zc;
	int             m_f;

	double FORTRAN_DIM(double x, double y) const;
	double knifeEdgeAttenuation(double v2) const;
	double heightGain(double x, double pk) const;
	double h0f(double r, double et) const;
	double ahd(double td) const;
	double diffractionAttenuation(double d);
	double scatterAttenuation(double d);
	double qerfi(double q) const;
	void   qlrps(double zsys, double en0, int ipol, double eps, double sgm);
	double abq(const complex<double>& r) const;
	double losAttenuation(double d);
	void   lrProp(double d);
	double curve(double c1, double c2, double x1, double x2, double x3, double de) const;
	double avar();
	void   horizons(double pfl[]);
	void   lsqf(double z[], double x1, double x2, double& z0, double& zn) const;
	double quantile(int nn, double a[], int ir) const;
	double dlthx(double pfl[], double x1, double x2) const;
	void   qlrpfl(double pfl[]);
};

#endif
