/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Location.h"

#include <wx/tokenzr.h>

CLocation::CLocation() :
m_latitude(0.0),
m_longitude(0.0),
m_format(FORMAT_LATLON),
m_valid(false)
{
}

CLocation::CLocation(double latitude, double longitude) :
m_latitude(0.0),
m_longitude(0.0),
m_format(FORMAT_LATLON),
m_valid(false)
{
	setLatLon(latitude, longitude);
}

CLocation::CLocation(EFormat format, const wxString& location) :
m_latitude(0.0),
m_longitude(0.0),
m_format(FORMAT_LATLON),
m_valid(false)
{
	setLocation(format, location);
}

CLocation::CLocation(const CLocation& location) :
m_latitude(location.m_latitude),
m_longitude(location.m_longitude),
m_format(location.m_format),
m_valid(location.m_valid)
{
}

CLocation::~CLocation()
{
}

bool CLocation::setLocation(EFormat format, const wxString& location)
{
	switch (format) {
		case FORMAT_LATLON:
			return setLatLon(location);

		case FORMAT_LOCATOR:
			return setLocator(location);

		case FORMAT_NGR:
			return setNGR(location);

		case FORMAT_UTM:
			return setUTM(location);

		default:
			return false;
	}
}

bool CLocation::setLatLon(const wxString& latLon)
{
	m_format = FORMAT_LATLON;

	wxStringTokenizer str(latLon);

	if (str.CountTokens() != 2) {
		m_valid = false;
		return m_valid;
	}

	wxString txtLat = str.GetNextToken();
	wxString txtLon = str.GetNextToken();

	wxString ns = txtLat.Right(1);
	wxString ew = txtLon.Right(1);

	double lat;
	txtLat.ToDouble(&lat);

	double lon;
	txtLon.ToDouble(&lon);

	if (ns.IsSameAs(wxT("S"), false))
		lat *= -1.0;

	if (ew.IsSameAs(wxT("W"), false))
		lon *= -1.0;

	if (lat >= 90.0 || lat <= -90.0) {
		m_valid = false;
		return m_valid;
	}

	if (lon >= 180.0 && lon < 360.0)
		lon -= 180.0;

	if (lon <= -180.0 && lon > -360.0)
		lon += 180.0;

	if (lon >= 180.0 || lon <= -180.0) {
		m_valid = false;
		return m_valid;
	}

	m_latitude  = lat;
	m_longitude = lon;

	m_valid = true;

	return true;
}

bool CLocation::setLocator(const wxString& locator)
{
	m_valid = false;

	wxString loc = locator;
	loc.MakeUpper();

	if (loc.Length() != 6 && loc.Length() != 8 && loc.Length() != 10)
		return m_valid;

	if (loc.GetChar(0) < wxT('A') || loc.GetChar(0) > wxT('R') ||
	    loc.GetChar(1) < wxT('A') || loc.GetChar(1) > wxT('R') ||
	    loc.GetChar(2) < wxT('0') || loc.GetChar(2) > wxT('9') ||
	    loc.GetChar(3) < wxT('0') || loc.GetChar(3) > wxT('9') ||
	    loc.GetChar(4) < wxT('A') || loc.GetChar(4) > wxT('X') ||
	    loc.GetChar(5) < wxT('A') || loc.GetChar(5) > wxT('X'))
		return m_valid;

	if (loc.Length() == 8 &&
	   (loc.GetChar(6) < wxT('0') || loc.GetChar(6) > wxT('9') ||
	    loc.GetChar(7) < wxT('0') || loc.GetChar(7) > wxT('9')))
		return m_valid;

	if (loc.Length() == 10 &&
	   (loc.GetChar(8) < wxT('A') || loc.GetChar(8) > wxT('X') ||
	    loc.GetChar(9) < wxT('A') || loc.GetChar(9) > wxT('X')))
		return m_valid;

	m_longitude = -180.0 + FIELD_WIDTH   * (loc.GetChar(0) - wxT('A')) +
			       SQUARE_WIDTH  * (loc.GetChar(2) - wxT('0')) +
			       SUB_WIDTH     * (loc.GetChar(4) - wxT('A'));

	m_latitude  = -90.0  + FIELD_HEIGHT  * (loc.GetChar(1) - wxT('A')) +
			       SQUARE_HEIGHT * (loc.GetChar(3) - wxT('0')) +
			       SUB_HEIGHT    * (loc.GetChar(5) - wxT('A'));

	if (loc.length() == 10) {
		m_longitude += SUB_SUB_WIDTH  * (loc.GetChar(6) - wxT('0')) + SUB_SUB_SUB_WIDTH  * (loc.GetChar(8) - wxT('A')) + SUB_SUB_SUB_WIDTH / 2.0;
		m_latitude  += SUB_SUB_HEIGHT * (loc.GetChar(7) - wxT('0')) + SUB_SUB_SUB_HEIGHT * (loc.GetChar(9) - wxT('A')) + SUB_SUB_SUB_HEIGHT / 2.0;
	} else if (loc.Length() == 8) {
		m_longitude += SUB_SUB_WIDTH  * (loc.GetChar(6) - wxT('0')) + SUB_SUB_WIDTH / 2.0;
		m_latitude  += SUB_SUB_HEIGHT * (loc.GetChar(7) - wxT('0')) + SUB_SUB_HEIGHT / 2.0;
	} else {
		m_longitude += SUB_SUB_WIDTH * 5.0;
		m_latitude  += SUB_SUB_HEIGHT * 5.0;
	}

	m_valid  = true;
	m_format = FORMAT_LOCATOR;

	return m_valid;
}

bool CLocation::setNGR(const wxString& ngr)
{
	m_valid = false;

	wxString NGR = ngr;
	NGR.MakeUpper();

	if (NGR.Length() != 2 && NGR.Length() != 4 && NGR.Length() != 6 &&
	    NGR.Length() != 8 && NGR.Length() != 10)
		return m_valid;

	wxString letters = NGR.Left(2);
	wxString numbers = NGR.Mid(2);

	wxChar first = letters.GetChar(0);
	if (first != wxT('S') && first != wxT('T') && first != wxT('N') &&
	    first != wxT('H') && first != wxT('O'))
		return m_valid;

	wxChar second = letters.GetChar(1);
	if (second < wxT('A') || second > wxT('Z') || second == wxT('I'))
		return m_valid;

	for (int i = 0; i < int(numbers.Length()); i++) {
		wxChar c = numbers.GetChar(i);

		if (c < wxT('0') || c > wxT('9'))
			return m_valid;
	}

	int s500 = letters.GetChar(0) - wxT('A');
	if (s500 > 8) s500--;

	int northings = 500000 * (3 - s500 / 5);
	int eastings  = 500000 * (s500 % 5 - 2);

	int s100 = letters.GetChar(1) - wxT('A');
	if (s100 > 8) s100--;

	northings += 100000 * (4 - s100 / 5);
	eastings  += 100000 * (s100 % 5);

	if (numbers.Length() > 0) {
		int scale = 0;

		switch (numbers.Length()) {
			case 2:
				scale = 10000;
				break;
			case 4:
				scale = 1000;
				break;
			case 6:
				scale = 100;
				break;
			case 8:
				scale = 10;
				break;
		}

		long n, e;
		numbers.Right(numbers.Length() / 2).ToLong(&n);
		numbers.Left(numbers.Length() / 2).ToLong(&e);

		eastings  += e * scale;
		northings += n * scale;
	}

	double t1 = (double(northings) / 1000.0 + 5548.79) / 6371.28;
	double t2 = 2.0 * ::atan2(::exp((double(eastings) / 1000.0 - 400.0) / 6389.70), 1.0);

	m_longitude = DEG(::atan2(-::cos(t2) / (::cos(t1) * ::sin(t2)), 1.0)) - 2.0;

	double n = ::sin(t2) * ::sin(t1);
	m_latitude = DEG(::atan2(n / ::sqrt(1.0 - n * n), 1.0));

	m_valid  = true;
	m_format = FORMAT_NGR;

	return m_valid;
}

bool CLocation::setUTM(const wxString& utm)
{
	m_format = FORMAT_UTM;
	m_valid  = false;

	return false;
}

bool CLocation::setLatLon(double latitude, double longitude)
{
	m_latitude  = latitude;
	m_longitude = longitude;
	m_valid     = false;

	if (m_latitude > 90.0 || m_latitude < -90.0)
		return m_valid;

	if (m_longitude > 180.0 || m_longitude < -180.0)
		return m_valid;

	m_valid  = true;
	m_format = FORMAT_LATLON;

	return m_valid;
}

bool CLocation::isValid(EFormat format) const
{
	switch (format) {
		case FORMAT_NGR:
			if (!m_valid)
				return false;

			return m_latitude >= 49.0  && m_latitude <= 61.0 &&
			       m_longitude >= -8.0 && m_longitude <= 1.8;

		case FORMAT_LATLON:
		case FORMAT_LOCATOR:
		case FORMAT_UTM:
			return m_valid;

		default:
			return false;
	}
}

bool CLocation::isSameAs(const CLocation& location) const
{
	return (m_latitude  == location.m_latitude &&
		m_longitude == location.m_longitude);
}

EFormat CLocation::getFormat() const
{
	return m_format;
}

wxString CLocation::getLocation(EFormat format) const
{
	switch (format) {
		case FORMAT_LATLON:
			return getLatLon();

		case FORMAT_LOCATOR:
			return getLocator();

		case FORMAT_NGR:
			return getNGR();

		case FORMAT_UTM:
			return getUTM();

		default:
			return wxT("[Invalid]");
	}
}

wxString CLocation::getLatLon() const
{
	if (!m_valid)
		return wxT("[Invalid Lat/Lon]");

	wxString text;

	wxChar ns = (m_latitude >= 0.0) ? wxT('N') : wxT('S');
	wxChar ew = (m_longitude >= 0.0) ? wxT('E') : wxT('W');

	text.Printf(wxT("%.5f%c %.5f%c"), ::fabs(m_latitude), ns, ::fabs(m_longitude), ew);

	return text;
}

wxString CLocation::getLocator() const
{
	if (!m_valid)
		return wxT("[Invalid Locator]");

	double latitude  = m_latitude + 90.0;
	double longitude = m_longitude + 180.0;

	wxString locator;

	int ilong = int(longitude / FIELD_WIDTH);
	locator.Append(ilong + wxT('A'));
	longitude -= double(ilong) * FIELD_WIDTH;

	int ilat = int(latitude / FIELD_HEIGHT);
	locator.Append(ilat + wxT('A'));
	latitude -= double(ilat) * FIELD_HEIGHT;

	ilong = int(longitude / SQUARE_WIDTH);
	locator.Append(ilong + wxT('0'));
	longitude -= double(ilong) * SQUARE_WIDTH;

	ilat = int(latitude / SQUARE_HEIGHT);
	locator.Append(ilat + wxT('0'));
	latitude -= double(ilat) * SQUARE_HEIGHT;

	ilong = int(longitude / SUB_WIDTH);
	locator.Append(ilong + wxT('A'));
	longitude -= double(ilong) * SUB_WIDTH;

	ilat = int(latitude / SUB_HEIGHT);
	locator.Append(ilat + wxT('A'));
	latitude -= double(ilat) * SUB_HEIGHT;

	ilong = int(longitude / SUB_SUB_WIDTH);
	locator.Append(ilong + wxT('0'));
	longitude -= double(ilong) * SUB_SUB_WIDTH;

	ilat = int(latitude / SUB_SUB_HEIGHT);
	locator.Append(ilat + wxT('0'));
	latitude -= double(ilat) * SUB_SUB_HEIGHT;

	ilong = int(longitude / SUB_SUB_SUB_WIDTH);
	locator.Append(ilong + wxT('a'));

	ilat = int(latitude / SUB_SUB_SUB_HEIGHT);
	locator.Append(ilat + wxT('a'));

	return locator;
}

wxString CLocation::getNGR() const
{
	if (!m_valid)
		return wxT("[Invalid NGR]");

	if (m_latitude < 49.0  || m_latitude > 61.0 ||
	    m_longitude < -8.0 || m_longitude > 1.8)
		return wxT("[Invalid NGR]");

	double e = RAD(m_longitude + 2.0);
	double n = RAD(m_latitude);

	double e1 = ::cos(n) * ::sin(e);
	double e2 = M_PI / 4.0 + ::atan2(e1 / ::sqrt(1.0 - e1 * e1), 1.0) / 2.0;
	double e3 = 6389.7 * ::log(::sin(e2) / ::cos(e2)) + 400.0;
	double n1 = 6371.28 * ::atan2(::sin(n) / (::cos(n) * ::cos(e)), 1.0) - 5548.79;

	int eastings  = int(e3 * 1000.0 + 0.5);
	int northings = int(n1 * 1000.0 + 0.5);

	eastings  += 1000000;
	northings += 500000;

	if (eastings  < 0 || eastings  >= 2500000 ||
	    northings < 0 || northings >= 2500000)
		return wxT("[Invalid NGR]");

	int s500 = eastings / 500000 + 5 * (4 - northings / 500000);
	if (s500 > 7)
		s500++;
	eastings %= 500000;
	northings %= 500000;

	int s100 = eastings / 100000 + 5 * (4 - northings / 100000);
	if (s100 > 7)
		s100++;
	eastings %= 100000;
	northings %= 100000;

	wxString ngr;
	ngr.Printf(wxT("%c%c%04d%04d"), s500 + wxT('A'), s100 + wxT('A'), eastings / 10, northings / 10);

	return ngr;
}

wxString CLocation::getUTM() const
{
	if (!m_valid)
		return wxT("[Invalid UTM]");

	return wxT("[Invalid UTM]");
}

void CLocation::getLatLon(double& latitude, double& longitude) const
{
	latitude  = m_latitude;
	longitude = m_longitude;
}

double CLocation::getLatitude() const
{
	return m_latitude;
}

double CLocation::getLongitude() const
{
	return m_longitude;
}
