/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	Location_H
#define	Location_H

#include <wx/string.h>

const double FIELD_WIDTH        = 20.0;				/* A - R */
const double FIELD_HEIGHT       = 10.0;				/* A - R */
const double SQUARE_WIDTH       = FIELD_WIDTH / 10.0;		/* 0 - 9 */
const double SQUARE_HEIGHT      = FIELD_HEIGHT / 10.0;		/* 0 - 9 */
const double SUB_WIDTH          = SQUARE_WIDTH / 24.0;		/* A - X */
const double SUB_HEIGHT         = SQUARE_HEIGHT / 24.0;		/* A - X */
const double SUB_SUB_WIDTH      = SUB_WIDTH / 10.0;		/* 0 - 9 */
const double SUB_SUB_HEIGHT     = SUB_HEIGHT / 10.0;		/* 0 - 9 */
const double SUB_SUB_SUB_WIDTH  = SUB_SUB_WIDTH / 24.0;		/* A - X */
const double SUB_SUB_SUB_HEIGHT = SUB_SUB_HEIGHT / 24.0;	/* A - X */

#include "Defs.h"

class CLocation {

    public:
	CLocation();
	CLocation(double latitude, double longitude);
	CLocation(EFormat format, const wxString& location);
	CLocation(const CLocation& location);
	virtual ~CLocation();

	virtual bool setLocation(EFormat format, const wxString& location);
	virtual bool setLatLon(double latitude, double longitude);

	virtual bool    isValid(EFormat format = FORMAT_LOCATOR) const;
	virtual bool    isSameAs(const CLocation& location) const;
	virtual EFormat getFormat() const;

	virtual wxString getLocation(EFormat format) const;
	virtual void     getLatLon(double& latitude, double& longitude) const;
	virtual double   getLatitude() const;
	virtual double   getLongitude() const;

    private:
	double  m_latitude;
	double  m_longitude;
	EFormat m_format;
	bool    m_valid;

	wxString getLatLon() const;
	wxString getLocator() const;
	wxString getNGR() const;
	wxString getUTM() const;
	
	bool setLatLon(const wxString& latLon);
	bool setLocator(const wxString& locator);
	bool setNGR(const wxString& ngr);
	bool setUTM(const wxString& utm);
};

#endif
