/*
 *   Copyright (C) 2002-2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Map.h"
#include "App.h"
#include "Coverage.h"
#include "Dialog.h"
#include "Terrain.h"
#include "3DMap.h"
#include "Horizon.h"
#include "PointDialog.h"
#include "HorizonDialog.h"
#include "TerrainDialog.h"
#include "CoverageDialog.h"
#include "PointFile.h"
#include "PointProperties.h"

#include <wx/progdlg.h>

const int NUM_COLOURS = 30;
const int SHADE_WIDTH = 255 / NUM_COLOURS;
const int SHADE_START = 255 - NUM_COLOURS * SHADE_WIDTH;
const int SHADE_TOP1  = NUM_COLOURS / 2;
const int SHADE_TOP2  = NUM_COLOURS / 3;

enum {
	Mouse_Map,
	Mouse_Terrain,
	Mouse_ReCentre
};

enum {
	Menu_Resolution_30 = 6382,
	Menu_Resolution_60,
	Menu_Resolution_90,
	Menu_Resolution_125,
	Menu_Resolution_250,
	Menu_Resolution_500,
	Menu_Resolution_1000,
	Menu_Resolution_2000,
	Menu_Resolution_4000,

	Menu_Type_Pseudo_3D1,
	Menu_Type_Pseudo_3D2,
	Menu_Type_Elevation,

	Menu_Coverage_On,
	Menu_Coverage_Off,

	Menu_Towns_On,
	Menu_Towns_Off,

	Menu_Sites_On,
	Menu_Sites_Off
};

BEGIN_EVENT_TABLE(CMap, wxPanel)
	EVT_MOTION(CMap::onMouseMove)
	EVT_PAINT(CMap::onPaint)
	EVT_LEFT_DOWN(CMap::onMouseLeftClick)
	EVT_RIGHT_DOWN(CMap::onMouseRightClick)
	EVT_MENU(Menu_Item_1,  CMap::onMenuSavePoint)
	EVT_MENU(Menu_Item_2,  CMap::onMenuSaveAsPoint)
	EVT_MENU(Menu_Item_3,  CMap::onMenuRemovePoint)
	EVT_MENU(Menu_Item_4,  CMap::onMenu3DMap)
	EVT_MENU(Menu_Item_5,  CMap::onMenuHorizon)
	EVT_MENU(Menu_Item_6,  CMap::onMenuStartTerrain)
	EVT_MENU(Menu_Item_8,  CMap::onMenuProperties)
	EVT_MENU(Menu_Item_10, CMap::onMenuNewPoint)
	EVT_MENU(Menu_Item_11, CMap::onMenuOpenPoint)
	EVT_MENU(Menu_Item_12, CMap::onMenuRemovePoints)
	EVT_MENU(Menu_Item_13, CMap::onMenuCoverage)
	EVT_MENU(Menu_Item_18, CMap::onMenuReCentre)
	EVT_MENU(Menu_Coverage_On,  CMap::onMenuCoverageSelect)
	EVT_MENU(Menu_Coverage_Off, CMap::onMenuCoverageSelect)
	EVT_MENU(Menu_Towns_On,  CMap::onMenuTowns)
	EVT_MENU(Menu_Towns_Off, CMap::onMenuTowns)
	EVT_MENU(Menu_Sites_On,  CMap::onMenuSites)
	EVT_MENU(Menu_Sites_Off, CMap::onMenuSites)
	EVT_MENU(Menu_Resolution_30,   CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_60,   CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_90,   CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_125,  CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_250,  CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_500,  CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_1000, CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_2000, CMap::onMenuResolution)
	EVT_MENU(Menu_Resolution_4000, CMap::onMenuResolution)
	EVT_MENU(Menu_Type_Pseudo_3D1, CMap::onMenuType)
	EVT_MENU(Menu_Type_Pseudo_3D2, CMap::onMenuType)
	EVT_MENU(Menu_Type_Elevation,  CMap::onMenuType)
END_EVENT_TABLE()

CMap::CMap(wxWindow* parent, const CLocation& centre, int resolution, EType type, bool towns, bool sites, int textSize) :
wxPanel(parent, -1, wxDefaultPosition, wxSize(WINDOW_WIDTH, WINDOW_HEIGHT)),
m_bitmap(NULL),
m_centre(centre),
m_resolution(resolution),
m_type(type),
m_towns(towns),
m_sites(sites),
m_textSize(textSize),
m_menu1(NULL),
m_menuCoverage(NULL),
m_menu2(NULL),
m_menuRes(NULL),
m_menuType(NULL),
m_menuTowns(NULL),
m_menuSites(NULL),
m_map(NULL),
m_colours(NULL),
m_coverage(),
m_ulLat(0.0),
m_ulLon(0.0),
m_brLat(0.0),
m_brLon(0.0),
m_stepLat(0.0),
m_stepLon(0.0),
m_maxHeight(-9999),
m_minHeight(9999),
m_mouseX(0),
m_mouseY(0),
m_mouseMode(Mouse_Map),
m_n(-1),
m_dynamicY(0),
m_height(0),
m_frequency(0),
m_polarisation(0),
m_climate(0),
m_ground(0),
m_confidence(0),
m_txPower(0),
m_txAntGain(0),
m_rxPower(0),
m_rxAntGain(0)
{
	m_bitmap   = new wxBitmap(WINDOW_WIDTH, WINDOW_HEIGHT);
	m_map      = new int[GRAPH_WIDTH * GRAPH_HEIGHT];
	m_colours  = new short[GRAPH_WIDTH * GRAPH_HEIGHT];

	for (int i = 0; i < MAX_COVERAGE_COUNT; i++)
		m_coverage[i] = new int[GRAPH_WIDTH * GRAPH_HEIGHT];

	for (int i = 0; i < (GRAPH_WIDTH * GRAPH_HEIGHT); i++) {
		m_map[i]     = 0;
		m_colours[i] = 0;
	}

	m_menuCoverage = new wxMenu();
	m_menuCoverage->AppendRadioItem(Menu_Coverage_On,  wxT("On"));
	m_menuCoverage->AppendRadioItem(Menu_Coverage_Off, wxT("Off"));

	m_menu1 = new wxMenu(wxT("Point Menu"));
	m_menu1->Append(Menu_Item_1, wxT("Save Point"));
	m_menu1->Append(Menu_Item_2, wxT("Save Point As..."));
	m_menu1->Append(Menu_Item_3, wxT("Remove Point"));
	m_menu1->AppendSeparator();
	m_menu1->Append(Menu_Item_4, wxT("3D Map"));
	m_menu1->Append(Menu_Item_5, wxT("Horizon"));
	m_menu1->Append(Menu_Item_6, wxT("Start Path Plot"));
	m_menu1->AppendSeparator();
	m_menu1->Append(Menu_Item_7, wxT("Coverage"), m_menuCoverage);
	m_menu1->AppendSeparator();
	m_menu1->Append(Menu_Item_8, wxT("Properties"));

	m_menuTowns = new wxMenu();
	m_menuTowns->AppendRadioItem(Menu_Towns_On,  wxT("On"));
	m_menuTowns->AppendRadioItem(Menu_Towns_Off, wxT("Off"));

	if (towns)
		m_menuTowns->Check(Menu_Towns_On, true);
	else
		m_menuTowns->Check(Menu_Towns_Off, true);

	m_menuSites = new wxMenu();
	m_menuSites->AppendRadioItem(Menu_Sites_On,  wxT("On"));
	m_menuSites->AppendRadioItem(Menu_Sites_Off, wxT("Off"));

	if (sites)
		m_menuSites->Check(Menu_Sites_On, true);
	else
		m_menuSites->Check(Menu_Sites_Off, true);

	m_menuRes = new wxMenu;
	m_menuRes->AppendRadioItem(Menu_Resolution_30,   wxT("30m"));
	m_menuRes->AppendRadioItem(Menu_Resolution_60,   wxT("60m"));
	m_menuRes->AppendRadioItem(Menu_Resolution_90,   wxT("90m"));
	m_menuRes->AppendRadioItem(Menu_Resolution_125,  wxT("125m"));
	m_menuRes->AppendRadioItem(Menu_Resolution_250,  wxT("250m"));
	m_menuRes->AppendRadioItem(Menu_Resolution_500,  wxT("500m"));
	m_menuRes->AppendRadioItem(Menu_Resolution_1000, wxT("1km"));
	m_menuRes->AppendRadioItem(Menu_Resolution_2000, wxT("2km"));
	m_menuRes->AppendRadioItem(Menu_Resolution_4000, wxT("4km"));

	switch (m_resolution) {
		case 30:
			m_menuRes->Check(Menu_Resolution_30, true);
			break;
		case 60:
			m_menuRes->Check(Menu_Resolution_60, true);
			break;
		case 90:
			m_menuRes->Check(Menu_Resolution_90, true);
			break;
		case 125:
			m_menuRes->Check(Menu_Resolution_125, true);
			break;
		case 250:
			m_menuRes->Check(Menu_Resolution_250, true);
			break;
		case 500:
			m_menuRes->Check(Menu_Resolution_500, true);
			break;
		case 1000:
			m_menuRes->Check(Menu_Resolution_1000, true);
			break;
		case 2000:
			m_menuRes->Check(Menu_Resolution_2000, true);
			break;
		case 4000:
			m_menuRes->Check(Menu_Resolution_4000, true);
			break;
		default:
			wxASSERT(false);
			break;
	}

	m_menuType = new wxMenu();
	m_menuType->AppendRadioItem(Menu_Type_Pseudo_3D1, wxT("Pseudo 3D 1"));
	m_menuType->AppendRadioItem(Menu_Type_Pseudo_3D2, wxT("Pseudo 3D 2"));
	m_menuType->AppendRadioItem(Menu_Type_Elevation, wxT("Elevation"));

	switch (type) {
		case TYPE_PSEUDO_3D_1:
			m_menuType->Check(Menu_Type_Pseudo_3D1, true);
			break;
		case TYPE_PSEUDO_3D_2:
			m_menuType->Check(Menu_Type_Pseudo_3D2, true);
			break;
		case TYPE_ELEVATION:
			m_menuType->Check(Menu_Type_Elevation, true);
			break;
		default:
			break;
	}

	m_menu2 = new wxMenu(wxT("Map Menu"));
	m_menu2->Append(Menu_Item_10, wxT("New Point..."));
	m_menu2->Append(Menu_Item_11, wxT("Open Point"));
	m_menu2->Append(Menu_Item_12, wxT("Remove all Points"));
	m_menu2->AppendSeparator();
	m_menu2->Append(Menu_Item_13, wxT("Coverage Plot..."));
	m_menu2->AppendSeparator();
	m_menu2->Append(Menu_Item_14, wxT("Towns"), m_menuTowns);
	m_menu2->Append(Menu_Item_15, wxT("Sites"), m_menuSites);
	m_menu2->Append(Menu_Item_16, wxT("Resolution"), m_menuRes);
	m_menu2->Append(Menu_Item_17, wxT("Map Type"), m_menuType);
	m_menu2->AppendSeparator();
	m_menu2->Append(Menu_Item_18, wxT("Re-Centre"));

	::wxGetApp().getCoveragePreferences(m_height, m_frequency, m_txPower, m_txAntGain, m_rxPower, m_rxAntGain, m_polarisation, m_climate, m_ground, m_confidence);

	createMapData();

	drawMap();

	if (m_towns)
		drawTowns();

	if (m_sites)
		drawSites();

	drawFixedData();
}

CMap::~CMap()
{
	delete m_bitmap;

	delete[] m_colours;
	delete[] m_map;

	for (int i = 0; i < MAX_COVERAGE_COUNT; i++)
		delete[] m_coverage[i];
}

void CMap::createMapData()
{
	CDEMCache* cache = ::wxGetApp().getDEMCache();

	wxBusyCursor busyCursor;

	wxProgressDialog dialog(wxT("Working..."), wxT("Creating Map Data"), 2 * GRAPH_HEIGHT, this);
	int progress = 0;

	wxASSERT(cache != NULL);

	for (int i = 0; i < MAX_COVERAGE_COUNT; i++) {
		for (int j = 0; j < (GRAPH_WIDTH * GRAPH_HEIGHT); j++)
			m_coverage[i][j] = VOID_DATA;
	}

	for (int i = 0; i < (GRAPH_WIDTH * GRAPH_HEIGHT); i++) {
		m_map[i]     = 0;
		m_colours[i] = 0;
	}

	double heightDeg = DEG(double(GRAPH_HEIGHT * m_resolution) / (R * 1000.0));
	double widthDeg  = DEG(double(GRAPH_WIDTH * m_resolution) / (R * 1000.0));

	m_maxHeight = -9999;
	m_minHeight = 9999;

	m_ulLat = m_centre.getLatitude() + heightDeg / 2.0;
	m_ulLon = m_centre.getLongitude() - widthDeg / 2.0;

	m_brLat = m_ulLat - heightDeg;
	m_brLon = m_ulLon + widthDeg;

	::wxGetApp().clearPoints(m_ulLat, m_ulLon, m_brLat, m_brLon);

	m_stepLat = (m_ulLat - m_brLat) / double(GRAPH_HEIGHT);
	m_stepLon = (m_brLon - m_ulLon) / double(GRAPH_WIDTH);

	for (int y = 0; y < GRAPH_HEIGHT; y++) {
		dialog.Update(progress++);

		double lat = yToLatitude(y);

		for (int x = 0; x < GRAPH_WIDTH; x++) {
			double lon = xToLongitude(x);

			int height = cache->getHeight(lat, lon);

			if (height == VOID_DATA) {
				m_map[y * GRAPH_WIDTH + x] = VOID_DATA;
			} else {
				if (height > m_maxHeight)
					m_maxHeight = height;

				if (height < m_minHeight)
					m_minHeight = height;

				m_map[y * GRAPH_WIDTH + x] = height;
			}
		}
	}

	int hgtRange = m_maxHeight - m_minHeight;
	double hgtBandWidth = double(hgtRange) / 250.0;
	double diff = double(hgtRange) / double(NUM_COLOURS * NUM_COLOURS);

	int scale[NUM_COLOURS];
	for (int i = 0; i < NUM_COLOURS; i++)
		scale[i] = m_minHeight + int(diff * double(i * i) + 0.5);

	for (int i = GRAPH_HEIGHT * GRAPH_WIDTH - 1; i >= GRAPH_WIDTH; i--) {
		if (i % GRAPH_WIDTH == 0)
			dialog.Update(progress++);

		if (m_map[i] != VOID_DATA) {
			switch (m_type) {
				case TYPE_ELEVATION: {
						int col = 0;
						int height = m_map[i];

						for (int j = NUM_COLOURS - 1; j >= 0; j--) {
							if (height > scale[j]) {
								col = j;
								break;
							}
						}

						if (col == 0 && height > 0)
							col = 1;

						m_colours[i] = col;
					}
					break;

				case TYPE_PSEUDO_3D_1: {
						int height1 = m_map[i];
						int height2 = m_map[i - GRAPH_WIDTH + 1];

						if (height1 == VOID_DATA)
							height1 = m_minHeight;

						if (height2 == VOID_DATA)
							height2 = m_minHeight;

						m_colours[i] = SHADE_TOP1 - int(double(height2 - height1) / hgtBandWidth);
					}
					break;

				case TYPE_PSEUDO_3D_2: {
						int height1 = m_map[i];
						int height2 = m_map[i - GRAPH_WIDTH + 1];

						if (height1 == VOID_DATA)
							height1 = m_minHeight;

						if (height2 == VOID_DATA)
							height2 = m_minHeight;

						m_colours[i] = SHADE_TOP2 - int(double(height2 - height1) / diff);
					}
					break;
			}

			if (m_colours[i] < 0)
				m_colours[i] = 0;

			if (m_colours[i] >= NUM_COLOURS)
				m_colours[i] = NUM_COLOURS - 1;
		} else {
			m_colours[i] = VOID_DATA;
		}
	}
}

void CMap::drawMap()
{
	wxASSERT(m_bitmap != NULL);

	wxBusyCursor busyCursor;

	wxProgressDialog dialog(wxT("Working..."), wxT("Drawing Map"), GRAPH_HEIGHT, this);

	wxColour noDataColour(0, 0, 150);
	wxPen noDataPen(noDataColour, 1, wxSOLID);

	wxPen pen[NUM_COLOURS][4];

	for (int i = 0; i < NUM_COLOURS; i++) {
		int n = getShade(i);

		for (int j = 0; j < 0x04; j++) {
			int r = 0, g = 0, b = 0;

			switch (j) {
				case 0x00:
					r = g = b = n;
					break;
				case 0x01:
					g = n;
					break;
				case 0x02:
					r = g = n;
					break;
				case 0x03:
					g = b = n;
					break;
			}

			wxColour colour = wxColour(r, g, b);
			pen[i][j] = wxPen(colour, 1, wxSOLID);
		}
	}

	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);
	dc.BeginDrawing();

	dc.SetClippingRegion(0, 0, GRAPH_WIDTH, GRAPH_HEIGHT);

	for (int y = 0; y < GRAPH_HEIGHT; y++) {
		dialog.Update(y);

		for (int x = 0; x < GRAPH_WIDTH; x++) {
			int colour = int(m_colours[y * GRAPH_WIDTH + x]);

			uchar cover = 0;
			uchar  mask = 0x01;

			for (int i = 0; i < MAX_COVERAGE_COUNT; i++, mask <<= 1)
				cover |= (m_coverage[i][y * GRAPH_WIDTH + x] >= 0) ? mask : 0x00;

			if (colour != VOID_DATA) {
				wxASSERT(colour >= 0 && colour < 20);
				wxASSERT(cover >= 0x00 && cover < 0x04);

				dc.SetPen(pen[colour][cover]);
				dc.DrawPoint(x, y);
			} else {
				wxASSERT(cover >= 0x00 && cover < 0x04);

				if (cover == 0x00)
					dc.SetPen(noDataPen);
				else
					dc.SetPen(pen[NUM_COLOURS - 1][cover]);
					
				dc.DrawPoint(x, y);
			}
		}
	}

	dc.SetPen(*wxRED_PEN);
	dc.SetTextForeground(*wxRED);

	wxFont font;
	font.SetPointSize(m_textSize);
	dc.SetFont(font);

	for (int i = 0; i < MAX_POINT_COUNT; i++) {
		CPoint* point = ::wxGetApp().getPoint(i);

		if (point == NULL)
			continue;

		int x = longitudeToX(point->getLocation().getLongitude());
		int y = latitudeToY(point->getLocation().getLatitude());

		dc.DrawRectangle(x - 3, y - 3, 6, 6);

		wxString name = point->getName();

		wxCoord width, height;
	        dc.GetTextExtent(name, &width, &height);

		dc.DrawText(name, x - width / 2, y + 3);
	}

	dc.EndDrawing();
}

void CMap::drawTowns()
{
	wxASSERT(m_bitmap != NULL);

	CTownList* townList = CTownList::getInstance();
	wxASSERT(townList != NULL);

	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);
	dc.BeginDrawing();

	dc.SetClippingRegion(0, 0, GRAPH_WIDTH, GRAPH_HEIGHT);

	dc.SetPen(*wxRED_PEN);
	dc.SetTextForeground(*wxRED);

	wxFont font;
	font.SetPointSize(m_textSize);
	dc.SetFont(font);

	CPoint* town = townList->getFirst(m_ulLat, m_ulLon, m_brLat, m_brLon);
	while (town != NULL) {
		CLocation location = town->getLocation();

		int x = longitudeToX(location.getLongitude());
		int y = latitudeToY(location.getLatitude());

		dc.DrawCircle(x, y, 2);

	        wxCoord width, height;
	        dc.GetTextExtent(town->getName(), &width, &height);

		dc.DrawText(town->getName(), x - width / 2, y + 2);

		town = townList->getNext();
	}

	dc.EndDrawing();
}

void CMap::drawSites()
{
	wxASSERT(m_bitmap != NULL);

	CSiteList* siteList = CSiteList::getInstance();
	wxASSERT(siteList != NULL);

	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);
	dc.BeginDrawing();

	dc.SetClippingRegion(0, 0, GRAPH_WIDTH, GRAPH_HEIGHT);

	dc.SetPen(*wxRED_PEN);
	dc.SetTextForeground(*wxRED);

	wxFont font;
	font.SetPointSize(m_textSize);
	dc.SetFont(font);

	CPoint* site = siteList->getFirst(m_ulLat, m_ulLon, m_brLat, m_brLon);
	while (site != NULL) {
		CLocation location = site->getLocation();

		int x = longitudeToX(location.getLongitude());
		int y = latitudeToY(location.getLatitude());

		wxPoint points[3];
		points[0] = wxPoint(x - 2, y + 1);
		points[1] = wxPoint(x + 2 ,y + 1);
		points[2] = wxPoint(x, y - 4);
		dc.DrawPolygon(3, points);

	        wxCoord width, height;
	        dc.GetTextExtent(site->getName(), &width, &height);

		dc.DrawText(site->getName(), x - width / 2, y + 2);

		site = siteList->getNext();
	}

	dc.EndDrawing();
}

void CMap::drawFixedData()
{
	wxASSERT(m_bitmap != NULL);

	int scale = 0, len = 0;

	for (int n = 0; n < 10; n++) {
		int scl = 0;

		switch (n) {
			case 0: scl = 1000;    break;
			case 1: scl = 2000;    break;
			case 2: scl = 5000;    break;
			case 3: scl = 10000;   break;
			case 4: scl = 20000;   break;
			case 5: scl = 50000;   break;
			case 6: scl = 100000;  break;
			case 7: scl = 200000;  break;
			case 8: scl = 500000;  break;
			case 9: scl = 1000000; break;
		}

		len = scl / m_resolution;

		if (len > 40) {
			scale = scl / 1000;
			break;
		}
	}

	int midX = GRAPH_WIDTH + INFO_WIDTH / 2;

	wxMemoryDC dc;
	dc.SelectObject(*m_bitmap);
	dc.BeginDrawing();

	dc.SetBrush(*wxGREY_BRUSH);
	dc.SetPen(*wxGREY_PEN);
	dc.DrawRectangle(GRAPH_WIDTH, 0, INFO_WIDTH, INFO_HEIGHT);

	dc.SetTextForeground(*wxBLACK);
	dc.SetBrush(*wxWHITE_BRUSH);
	dc.SetPen(*wxBLACK_PEN);

	dc.DrawLine(midX - len / 2, WINDOW_HEIGHT - 20, midX + len / 2, WINDOW_HEIGHT - 20);
	dc.DrawLine(midX - len / 2, WINDOW_HEIGHT - 20, midX - len / 2, WINDOW_HEIGHT - 25);
	dc.DrawLine(midX + len / 2, WINDOW_HEIGHT - 20, midX + len / 2, WINDOW_HEIGHT - 25);

	wxString text;
	text.Printf(wxT("%d km"), scale);

	wxCoord width, height;
	dc.GetTextExtent(text, &width, &height);

	dc.DrawText(text, midX - width / 2, WINDOW_HEIGHT - 35);

	dc.DrawLine(GRAPH_WIDTH + 5, WINDOW_HEIGHT - 55, GRAPH_WIDTH + INFO_WIDTH - 5, WINDOW_HEIGHT - 55);

	int x = GRAPH_WIDTH + 10;
	int y = WINDOW_HEIGHT - 270;
	
	dc.DrawLine(GRAPH_WIDTH + 5, y, GRAPH_WIDTH + INFO_WIDTH - 5, y);
	y += 20;

	dc.DrawText(wxT("Legend:"), x, y);
	y += 20;

	dc.SetPen(*wxRED_PEN);
	dc.DrawCircle(x + 7, y + 7, 2);

	dc.DrawText(wxT("Towns"), x + 50, y);
	y += 20;

	wxPoint points[3];
	points[0] = wxPoint(x + 5, y + 8);
	points[1] = wxPoint(x + 9, y + 8);
	points[2] = wxPoint(x + 7, y + 3);
	dc.DrawPolygon(3, points);

	dc.DrawText(wxT("Sites"), x + 50, y);
	y += 20;

	dc.DrawRectangle(x + 4, y + 2, 6, 6);

	dc.DrawText(wxT("Points"), x + 50, y);
	y += 20;

	drawGradient(dc, x, y, true, true, true);
	
	dc.DrawText(wxT("Terrain"), x + 50, y);
	y += 20;

	dc.DrawText(wxT("Coverage:"), x, y);
	y += 20;

	drawGradient(dc, x, y, false, true, false);
	
	dc.DrawText(wxT("Point 1"), x + 50, y);
	y += 20;

	drawGradient(dc, x, y, true, true, false);
	
	dc.DrawText(wxT("Point 2"), x + 50, y);
	y += 20;

	drawGradient(dc, x, y, false, true, true);
	
	dc.DrawText(wxT("Point 1 + 2"), x + 50, y);

	dc.SetPen(*wxBLACK_PEN);

	y = 15;
	dc.DrawText(wxT("Map centre:"), x, y);
	y += 15;

	dc.DrawText(m_centre.getLocation(FORMAT_LATLON), x, y);
	y += 15;

	text.Printf(wxT("Loc: %s"), m_centre.getLocation(FORMAT_LOCATOR).c_str());
	dc.DrawText(text, x, y);
	y += 15;

	if (m_centre.isValid(FORMAT_NGR)) {
		text.Printf(wxT("NGR: %s"), m_centre.getLocation(FORMAT_NGR).c_str());
		dc.DrawText(text, x, y);
		y += 15;
	}

	y += 5;

	dc.DrawText(wxT("Resolution:"), x, y);
	y += 15;

	text.Printf(wxT("%d m/pixel"), m_resolution);
	dc.DrawText(text, x, y);
	y += 25;

	dc.DrawLine(GRAPH_WIDTH + 5, y, GRAPH_WIDTH + INFO_WIDTH - 5, y);
	y += 15;

	m_dynamicY = y;

	dc.EndDrawing();
}

void CMap::onPaint(const wxPaintEvent& event)
{
	wxPaintDC dc(this);

	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

void CMap::onMouseMove(const wxMouseEvent& event)
{
	int x, y;
	event.GetPosition(&x, &y);

	if (x < 0 || x >= GRAPH_WIDTH || y < 0 || y >= GRAPH_HEIGHT) {
		wxClientDC dc(this);
		dc.BeginDrawing();

		dc.SetBrush(*wxGREY_BRUSH);
		dc.SetPen(*wxGREY_PEN);

		dc.DrawRectangle(GRAPH_WIDTH + 5, m_dynamicY, INFO_WIDTH - 10, 125);

		dc.EndDrawing();

		return;
	}

	double lat = yToLatitude(y);
	double lon = xToLongitude(x);

	if (lat != VOID_DATA && lon != VOID_DATA) {
		int height = m_map[y * GRAPH_WIDTH + x];

		int sig[MAX_COVERAGE_COUNT];
		for (int i = 0; i < MAX_COVERAGE_COUNT; i++)
			sig[i] = m_coverage[i][y * GRAPH_WIDTH + x];

		wxClientDC dc(this);
		dc.BeginDrawing();

		dc.SetBrush(*wxGREY_BRUSH);
		dc.SetPen(*wxGREY_PEN);

		x = GRAPH_WIDTH + 10;
		y = m_dynamicY;
		dc.DrawRectangle(GRAPH_WIDTH + 5, y, INFO_WIDTH - 10, 125);

		CLocation locator;
		locator.setLatLon(lat, lon);

		dc.SetTextForeground(*wxBLACK);
		dc.SetPen(*wxBLACK_PEN);

		dc.DrawText(wxT("Mouse location:"), x, y);
		y += 15;

		dc.DrawText(locator.getLocation(FORMAT_LATLON), x, y);
		y += 15;

		wxString text;
		text.Printf(wxT("Loc: %s"), locator.getLocation(FORMAT_LOCATOR).c_str());
		dc.DrawText(text, x, y);
		y += 15;

		if (locator.isValid(FORMAT_NGR)) {
			text.Printf(wxT("NGR: %s"), locator.getLocation(FORMAT_NGR).c_str());
			dc.DrawText(text, x, y);
			y += 15;
		}

		y += 5;

		if (height == VOID_DATA)
			text = wxT("Height: No Data");
		else
			text.Printf(wxT("Height: %dm ASL"), height);
		dc.DrawText(text, x, y);
		y += 15;

		switch (m_mouseMode) {
			case Mouse_Map:
			case Mouse_ReCentre:
				for (int i = 0; i < MAX_COVERAGE_COUNT; i++) {
					if (sig[i] == VOID_DATA)
						text.Printf(wxT("Point %d: No Data"), i + 1);
					else
						text.Printf(wxT("Point %d: %d dBm"), i + 1, sig[i]);
					dc.DrawText(text, x, y);
					y += 15;
				}
				break;

			case Mouse_Terrain: {
					wxASSERT(m_n != -1);
					CPoint* point = ::wxGetApp().getPoint(m_n);
					wxASSERT(point != NULL);

					CPath path(point->getLocation());
					path.setDest(locator);

					text.Printf(wxT("Az: %.1f deg"), path.getBearing());
					dc.DrawText(text, x, y);
					y += 15;

					text.Printf(wxT("Dist: %.1f km"), path.getDistance());
					dc.DrawText(text, x, y);
					y += 15;
				}
				break;
		}

		y += 10;

		dc.DrawLine(GRAPH_WIDTH + 5, y, GRAPH_WIDTH + INFO_WIDTH - 5, y);

		dc.EndDrawing();
	}
}

void CMap::onMouseLeftClick(const wxMouseEvent& event)
{
	int x, y;
	event.GetPosition(&x, &y);

	if (x < 0 || x >= GRAPH_WIDTH || y < 0 || y >= GRAPH_HEIGHT)
		return;

	switch (m_mouseMode) {
		case Mouse_Map:
			mouseMapClick(x, y);
			break;

		case Mouse_Terrain:
			m_mouseMode = Mouse_Map;
			mouseTerrainClick(x, y);
			break;

		case Mouse_ReCentre:
			m_mouseMode = Mouse_Map;
			mouseReCentreClick(x, y);
			break;

		default:
			wxASSERT(false);
			break;
	}
}

void CMap::mouseMapClick(int x, int y)
{
	m_mouseX = x;
	m_mouseY = y;

	wxCommandEvent commandEvent;
	onMenuNewPoint(commandEvent);
}

void CMap::mouseTerrainClick(int x, int y)
{
	wxASSERT(m_n != -1);

	SetCursor(wxNullCursor);

	int n = getNearestPoint(x, y);

	CLocation location2;
	int height2;
	int asl2;
	if (n != -1) {
		if (n == m_n) {
			wxMessageDialog dialog(this, wxT("The two points are the same !"), wxT("UKW Tools Error"), wxICON_ERROR);
			dialog.ShowModal();
			return;
		}

		CPoint* point = ::wxGetApp().getPoint(n);
		wxASSERT(point != NULL);

		location2 = point->getLocation();
		height2   = point->getHeight();
		asl2      = point->getASL();
	} else {
		double lat = yToLatitude(y);
		double lon = xToLongitude(x);

		location2.setLatLon(lat, lon);
		height2 = m_height;
		asl2    = VOID_DATA;
	}

	CPoint* point = ::wxGetApp().getPoint(m_n);
	m_n = -1;

	wxASSERT(point != NULL);

	EFormat format = point->getLocation().getFormat();

	CTerrainDialog dialog(this, location2, format, height2, m_frequency, m_polarisation, m_climate, m_ground, m_confidence, m_txPower, m_txAntGain, m_rxPower, m_rxAntGain);

	if (dialog.ShowModal() != wxID_OK)
		return;

	location2        = *dialog.getLocation();
	height2          = dialog.getHeight();
	int txPower      = dialog.getTXPower();
	int txAntGain    = dialog.getTXAntGain();
	int rxPower      = dialog.getRXPower();
	int rxAntGain    = dialog.getRXAntGain();
	int frequency    = dialog.getFrequency();
	int polarisation = dialog.getPolarisation();
	int climate      = dialog.getClimate();
	int ground       = dialog.getGround();
	int confidence   = dialog.getConfidence();

	CDialog* cdialog = new CDialog(this, wxT("Path Plot"));
	wxPanel* panel  = new CTerrain(cdialog, *point, location2, height2, asl2, frequency, polarisation, climate, ground, confidence, txPower, txAntGain, rxPower, rxAntGain);

	cdialog->setContent(panel);
	cdialog->Show();
}

void CMap::mouseReCentreClick(int x, int y)
{
	SetCursor(wxNullCursor);

	double lat = yToLatitude(y);
	double lon = xToLongitude(x);

	m_centre = CLocation(lat, lon);

	wxString fileName;
	CLocation centre;
	int resolution;
	EType type;
	bool towns, sites;

	::wxGetApp().getMap(fileName, centre, resolution, type, towns, sites);

	::wxGetApp().setMap(fileName, m_centre, resolution, type, towns, sites);

	createMapData();

	drawFixedData();

	refreshMap();
}

void CMap::onMouseRightClick(const wxMouseEvent& event)
{
	wxASSERT(m_menu1 != NULL);
	wxASSERT(m_menu2 != NULL);
	wxASSERT(m_menuCoverage != NULL);

	int x, y;
	event.GetPosition(&x, &y);

	if (x < 0 || x >= GRAPH_WIDTH || y < 0 || y >= GRAPH_HEIGHT)
		return;

	m_mouseX = x;
	m_mouseY = y;

	int n = getNearestPoint(x, y);

	if (n != -1) {
		CPoint* point = wxGetApp().getPoint(n);
		wxASSERT(point != NULL);
		bool coverage = point->getCoverage();

		if (coverage)
			m_menuCoverage->Check(Menu_Coverage_On, true);
		else
			m_menuCoverage->Check(Menu_Coverage_Off, true);

		PopupMenu(m_menu1, x, y);
	} else {
		PopupMenu(m_menu2, x, y);
	}
}

double CMap::xToLongitude(int x) const
{
	wxASSERT(x >= 0 || x < GRAPH_WIDTH);

	double lon = m_ulLon + double(x) * m_stepLon;

	if (lon > 180.0)
		lon -= 360.0;

	if (lon < -180.0)
		lon += 360.0;

	return lon;
}

double CMap::yToLatitude(int y) const
{
	wxASSERT(y >= 0 || y < GRAPH_HEIGHT);

	return m_ulLat - double(y) * m_stepLat;
}

int CMap::longitudeToX(double lon) const
{
	return int((lon - m_ulLon) / m_stepLon);
}

int CMap::latitudeToY(double lat) const
{
	return int((m_ulLat - lat) / m_stepLat);
}

void CMap::onMenuCoverage(const wxCommandEvent& event)
{
	int n = 0;

	for (int i = 0; i < MAX_POINT_COUNT; i++) {
		CPoint* point = ::wxGetApp().getPoint(i);

		if (point == NULL)
			continue;

		if (!point->getCoverage())
			continue;

		n++;
	}

	if (n > MAX_COVERAGE_COUNT) {
		wxMessageDialog dialog(this, wxT("Too many points selected for coverage plot"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	CCoverageDialog dialog(this, m_height, m_frequency, m_polarisation, m_climate, m_ground, m_confidence, m_txPower, m_txAntGain, m_rxPower, m_rxAntGain);

	if (dialog.ShowModal() != wxID_OK)
		return;

	m_height        = dialog.getHeight();
	m_frequency     = dialog.getFrequency();
	m_polarisation  = dialog.getPolarisation();
	m_climate       = dialog.getClimate();
	m_ground        = dialog.getGround();
	m_confidence    = dialog.getConfidence();
	m_txPower       = dialog.getTXPower();
	m_txAntGain     = dialog.getTXAntGain();
	m_rxPower       = dialog.getRXPower();
	m_rxAntGain     = dialog.getRXAntGain();

	wxBusyCursor busyCursor;

	CCoverage coverage(m_map, m_resolution, m_ulLat, m_ulLon, m_brLat, m_brLon);

	coverage.setITM(m_frequency, m_polarisation, m_climate, m_ground, m_confidence);
	coverage.setRFParams(m_txPower, m_txAntGain, m_rxPower, m_rxAntGain);

	int site = 0;

	for (int i = 0; i < MAX_POINT_COUNT; i++) {
		CPoint* point = ::wxGetApp().getPoint(i);

		if (point == NULL)
			continue;

		if (!point->getCoverage())
			continue;

		CLocation location = point->getLocation();
		int height         = point->getHeight();

		coverage.calculate(m_coverage[site], location, height, m_height);
		site++;
	}

	refreshMap();
}

void CMap::onMenuStartTerrain(const wxCommandEvent& event)
{
	m_n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(m_n != -1);

	m_mouseMode = Mouse_Terrain;

	SetCursor(wxCURSOR_CROSS);
}

void CMap::onMenu3DMap(const wxCommandEvent& event)
{
	int n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(n != -1);

	CPoint* point = ::wxGetApp().getPoint(n);

	wxASSERT(point != NULL);

	int mapSize;
	::wxGetApp().get3DMapPreferences(mapSize);

	CDialog* dialog = new CDialog(this, wxT("3D Map"));
	wxPanel* panel  = new C3DMap(dialog, *point, mapSize);

	dialog->setContent(panel);
	dialog->Show();
}

void CMap::onMenuHorizon(const wxCommandEvent& event)
{
	int n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(n != -1);

	CPoint* point = ::wxGetApp().getPoint(n);

	wxASSERT(point != NULL);

	int distance;
	double minEle, maxEle, k;
	::wxGetApp().getHorizonPreferences(distance, minEle, maxEle, k);

	CHorizonDialog hDialog(this, distance, minEle, maxEle, k);

	if (hDialog.ShowModal() != wxID_OK)
		return;

	distance = hDialog.getDistance();
	minEle   = hDialog.getMinimumElevation();
	maxEle   = hDialog.getMaximumElevation();
	k        = hDialog.getK();
	int startAz = hDialog.getStartAngle();
	int endAz   = hDialog.getEndAngle();


	CDialog* dialog = new CDialog(this, wxT("Horizon View"));
	wxPanel* panel  = new CHorizon(dialog, *point, distance, startAz, endAz, minEle, maxEle, k);

	dialog->setContent(panel);
	dialog->Show();
}

void CMap::onMenuCoverageSelect(const wxCommandEvent& event)
{
	wxASSERT(m_menuCoverage != NULL);

	int n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(n != -1);

	CPoint* point = ::wxGetApp().getPoint(n);

	wxASSERT(point != NULL);

	switch (event.GetId()) {
		case Menu_Coverage_On:
			m_menuCoverage->Check(Menu_Coverage_On, true);
			point->setCoverage(true);
			break;
		case Menu_Coverage_Off:
			m_menuCoverage->Check(Menu_Coverage_Off, true);
			point->setCoverage(false);
			break;
		default:
			wxASSERT(false);
			break;
	}
}

void CMap::onMenuProperties(const wxCommandEvent& event)
{
	int n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(n != -1);

	CPoint* point = ::wxGetApp().getPoint(n);

	wxASSERT(point != NULL);

	CPointProperties dialog(this, *point);
	dialog.ShowModal();
}

void CMap::onMenuNewPoint(const wxCommandEvent& event)
{
	EFormat format;
	int resolution, textSize;
	EType type;
	bool towns, sites;
	::wxGetApp().getMapPreferences(format, resolution, type, towns, sites, textSize);

	double lat = yToLatitude(m_mouseY);
	double lon = xToLongitude(m_mouseX);

	CLocation location(lat, lon);

	int height, frequency, txPower, txAntGain, rxPower, rxAntGain, polarisation, climate, ground, confidence;
	::wxGetApp().getCoveragePreferences(height, frequency, txPower, txAntGain, rxPower, rxAntGain, polarisation, climate, ground, confidence);

	CPointDialog dialog(this, location, format, height, VOID_DATA);

	if (dialog.ShowModal() != wxID_OK)
		return;

	wxString name = dialog.getName();
	location      = *dialog.getLocation();
	height        = dialog.getHeight();
	int asl       = dialog.getASL();

	int x = longitudeToX(location.getLongitude());
	int y = latitudeToY(location.getLatitude());

	if (x < 0 || x >= GRAPH_WIDTH || y < 0 || y >= GRAPH_HEIGHT) {
		wxMessageDialog dialog(this, wxT("This point is outside of this map"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	CPoint* point = new CPoint(name, location, height, asl);

	int n = ::wxGetApp().setPoint(point);

	if (n == -1) {
		wxMessageDialog dialog(this, wxT("No space to store this point, please remove one first"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	refreshMap();
}

void CMap::onMenuRemovePoint(const wxCommandEvent& event)
{
	int n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(n != -1);

	CPoint* point = ::wxGetApp().getPoint(n);

	wxASSERT(point != NULL);

	wxString text;
	text.Printf(wxT("Do you want to remove %s from the map"), point->getName().c_str());

	wxMessageDialog dialog(this, text, wxT("UKW Tools Question"), wxICON_QUESTION | wxYES_NO);

	if (dialog.ShowModal() != wxID_YES)
		return;

	::wxGetApp().removePoint(n);

	refreshMap();
}

void CMap::onMenuRemovePoints(const wxCommandEvent& event)
{
	int n = ::wxGetApp().countPoints();

	if (n == 0) {
		wxMessageDialog dialog(this, wxT("There are no points to remove"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	wxMessageDialog dialog(this, wxT("Do you want to remove all of the points"), wxT("UKW Tools Question"), wxICON_QUESTION | wxYES_NO);

	if (dialog.ShowModal() != wxID_YES)
		return;

	::wxGetApp().clearPoints();

	refreshMap();
}

void CMap::onMenuOpenPoint(const wxCommandEvent& event)
{
	wxString srtmPath, globePath, gtopo30Path, dataPath, mapQTHPath;
	EDTEM dtem;

	::wxGetApp().getGeneralPreferences(srtmPath, globePath, gtopo30Path, dtem, dataPath, mapQTHPath);

	wxFileDialog dialog(this, wxT("Open Point File"), mapQTHPath, wxEmptyString, wxT("QTH Files (*.qth;*.QTH)|*.qth;*.QTH"));
	dialog.SetStyle(wxOPEN);

	if (dialog.ShowModal() != wxID_OK)
		return;

	wxString path = dialog.GetPath();

	wxString name;
	CLocation location;
	int height;
	int asl;

	CPointFile file;
	bool ret = file.read(path, name, location, height, asl);

	if (!ret) {
		wxMessageDialog dialog(this, wxT("Cannot open or parse the Point file"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	int x = longitudeToX(location.getLongitude());
	int y = latitudeToY(location.getLatitude());

	if (x < 0 || x >= GRAPH_WIDTH || y < 0 || y >= GRAPH_HEIGHT) {
		wxMessageDialog dialog(this, wxT("This point is outside of this map"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	CPoint* point = new CPoint(name, path, location, height, asl);

	int n = ::wxGetApp().setPoint(point);

	if (n == -1) {
		wxMessageDialog dialog(this, wxT("No space to store this point, please remove one first"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	refreshMap();
}

void CMap::onMenuSavePoint(const wxCommandEvent& event)
{
	int n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(n != -1);

	CPoint* point = ::wxGetApp().getPoint(n);

	wxASSERT(point != NULL);

	if (point->getFileName().IsEmpty()) {
		onMenuSaveAsPoint(event);
		return;
	}

	CPointFile file;
	bool ret = file.write(point->getFileName(), point->getName(), point->getLocation(), point->getHeight(), point->getASL());

	if (!ret) {
		wxMessageDialog dialog(this, wxT("Cannot write the Point file"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}
}

void CMap::onMenuSaveAsPoint(const wxCommandEvent& event)
{
	int n = getNearestPoint(m_mouseX, m_mouseY);

	wxASSERT(n != -1);

	CPoint* point = ::wxGetApp().getPoint(n);

	wxASSERT(point != NULL);

	wxString srtmPath, globePath, gtopo30Path, dataPath, mapQTHPath;
	EDTEM dtem;

	::wxGetApp().getGeneralPreferences(srtmPath, globePath, gtopo30Path, dtem, dataPath, mapQTHPath);

	wxFileDialog dialog(this, wxT("Save Point File"), mapQTHPath, wxEmptyString, wxT("QTH Files (*.qth;*.QTH)|*.qth;*.QTH"));
	dialog.SetStyle(wxSAVE | wxOVERWRITE_PROMPT);

	if (dialog.ShowModal() != wxID_OK)
		return;

	wxString path = dialog.GetPath();

	CPointFile file;
	bool ret = file.write(path, point->getName(), point->getLocation(), point->getHeight(), point->getASL());

	if (!ret) {
		wxMessageDialog dialog(this, wxT("Cannot write the Point file"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	point->setFileName(path);
}

void CMap::onMenuTowns(const wxCommandEvent& event)
{
	wxASSERT(m_menuTowns != NULL);

	m_towns = event.GetId() == Menu_Towns_On; 

	m_menuTowns->Check(event.GetId(), true);

	wxString fileName;
	CLocation centre;
	int resolution;
	EType type;
	bool towns, sites;

	::wxGetApp().getMap(fileName, centre, resolution, type, towns, sites);

	::wxGetApp().setMap(fileName, centre, resolution, type, m_towns, sites);

	refreshMap();
}

void CMap::onMenuSites(const wxCommandEvent& event)
{
	wxASSERT(m_menuSites != NULL);

	m_sites = event.GetId() == Menu_Sites_On; 

	m_menuSites->Check(event.GetId(), true);

	wxString fileName;
	CLocation centre;
	int resolution;
	EType type;
	bool towns, sites;

	::wxGetApp().getMap(fileName, centre, resolution, type, towns, sites);

	::wxGetApp().setMap(fileName, centre, resolution, type, towns, m_sites);

	refreshMap();
}

void CMap::onMenuResolution(const wxCommandEvent& event)
{
	wxASSERT(m_menuRes != NULL);

	switch (event.GetId()) {
		case Menu_Resolution_30:
			m_resolution = 30;
			break;
		case Menu_Resolution_60:
			m_resolution = 60;
			break;
		case Menu_Resolution_90:
			m_resolution = 90;
			break;
		case Menu_Resolution_125:
			m_resolution = 125;
			break;
		case Menu_Resolution_250:
			m_resolution = 250;
			break;
		case Menu_Resolution_500:
			m_resolution = 500;
			break;
		case Menu_Resolution_1000:
			m_resolution = 1000;
			break;
		case Menu_Resolution_2000:
			m_resolution = 2000;
			break;
		case Menu_Resolution_4000:
			m_resolution = 4000;
			break;
		default:
			wxASSERT(false);
			break;
	}

	m_menuRes->Check(event.GetId(), true);

	wxString fileName;
	CLocation centre;
	int resolution;
	EType type;
	bool towns, sites;

	::wxGetApp().getMap(fileName, centre, resolution, type, towns, sites);

	::wxGetApp().setMap(fileName, centre, m_resolution, type, towns, sites);

	createMapData();

	drawFixedData();

	refreshMap();
}

void CMap::onMenuType(const wxCommandEvent& event)
{
	wxASSERT(m_menuType != NULL);

	switch (event.GetId()) {
		case Menu_Type_Pseudo_3D1:
			m_type = TYPE_PSEUDO_3D_1;
			break;
		case Menu_Type_Pseudo_3D2:
			m_type = TYPE_PSEUDO_3D_2;
			break;
		case Menu_Type_Elevation:
			m_type = TYPE_ELEVATION;
			break;
		default:
			wxASSERT(false);
			break;
	}

	m_menuType->Check(event.GetId(), true);

	wxString fileName;
	CLocation centre;
	int resolution;
	EType type;
	bool towns, sites;

	::wxGetApp().getMap(fileName, centre, resolution, type, towns, sites);

	::wxGetApp().setMap(fileName, centre, resolution, m_type, towns, sites);

	createMapData();

	drawFixedData();

	refreshMap();
}

void CMap::onMenuReCentre(const wxCommandEvent& event)
{
	m_mouseMode = Mouse_ReCentre;

	SetCursor(wxCURSOR_CROSS);
}

int CMap::getNearestPoint(int x1, int y1) const
{
	for (int i = 0; i < MAX_POINT_COUNT; i++) {
		CPoint* point = ::wxGetApp().getPoint(i);

		if (point == NULL)
			continue;

		int y2 = latitudeToY(point->getLocation().getLatitude());
		int x2 = longitudeToX(point->getLocation().getLongitude());

		if (::abs(x1 - x2) < 6 && ::abs(y1 - y2) < 6)
			return i;
	}

	return -1;
}

void CMap::refreshMap()
{
	drawMap();

	if (m_towns)
		drawTowns();

	if (m_sites)
		drawSites();

	wxClientDC dc(this);
	dc.BeginDrawing();
	dc.DrawBitmap(*m_bitmap, 0, 0, false);
	dc.EndDrawing();
}

int CMap::getShade(int n) const
{
	wxASSERT(n >= 0 && n < NUM_COLOURS);

	return n * SHADE_WIDTH + SHADE_START;
}

void CMap::drawGradient(wxDC& dc, int x, int y, bool r, bool g, bool b)
{
	int n1 = getShade(NUM_COLOURS / 2);
	int n2 = getShade(NUM_COLOURS * 2 / 3);
	int n3 = getShade(NUM_COLOURS - 1);

	int r1 = (r) ? n1 : 0; int g1 = (g) ? n1 : 0; int b1 = (b) ? n1 : 0; 
	int r2 = (r) ? n2 : 0; int g2 = (g) ? n2 : 0; int b2 = (b) ? n2 : 0;  
	int r3 = (r) ? n3 : 0; int g3 = (g) ? n3 : 0; int b3 = (b) ? n3 : 0;  

	wxColour colour = wxColour(r1, g1, b1);
	wxPen    pen    = wxPen(colour, 1, wxSOLID);
	wxBrush  brush  = wxBrush(colour, wxSOLID);

	dc.SetPen(pen);
	dc.SetBrush(brush);
	dc.DrawRectangle(x, y, 15, 15);

	x += 15;

	colour = wxColour(r2, g2, b2);
	pen    = wxPen(colour, 1, wxSOLID);
	brush  = wxBrush(colour, wxSOLID);

	dc.SetPen(pen);
	dc.SetBrush(brush);
	dc.DrawRectangle(x, y, 15, 15);

	x += 15;

	colour = wxColour(r3, g3, b3);
	pen    = wxPen(colour, 1, wxSOLID);
	brush  = wxBrush(colour, wxSOLID);

	dc.SetPen(pen);
	dc.SetBrush(brush);
	dc.DrawRectangle(x, y, 15, 15);
}
