/*
 *   Copyright (C) 2002-2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	Map_H
#define	Map_H

#include <wx/wx.h>

#include "Defs.h"
#include "DEMCache.h"
#include "Location.h"
#include "TownList.h"
#include "SiteList.h"

class CMap : public wxPanel {

    public:
	CMap(wxWindow* parent, const CLocation& location, int resolution, EType type, bool towns, bool sites, int textSize);
	virtual ~CMap();

	virtual void onPaint(const wxPaintEvent& event);
	virtual void onMouseMove(const wxMouseEvent& event);
	virtual void onMouseLeftClick(const wxMouseEvent& event);
	virtual void onMouseRightClick(const wxMouseEvent& event);
	virtual void onMenu3DMap(const wxCommandEvent& event);
	virtual void onMenuHorizon(const wxCommandEvent& event);
	virtual void onMenuStartTerrain(const wxCommandEvent& event);
	virtual void onMenuCoverageSelect(const wxCommandEvent& event);
	virtual void onMenuProperties(const wxCommandEvent& event);
	virtual void onMenuNewPoint(const wxCommandEvent& event);
	virtual void onMenuOpenPoint(const wxCommandEvent& event);
	virtual void onMenuRemovePoint(const wxCommandEvent& event);
	virtual void onMenuRemovePoints(const wxCommandEvent& event);
	virtual void onMenuSavePoint(const wxCommandEvent& event);
	virtual void onMenuSaveAsPoint(const wxCommandEvent& event);
	virtual void onMenuCoverage(const wxCommandEvent& event);
	virtual void onMenuTowns(const wxCommandEvent& event);
	virtual void onMenuSites(const wxCommandEvent& event);
	virtual void onMenuResolution(const wxCommandEvent& event);
	virtual void onMenuType(const wxCommandEvent& event);
	virtual void onMenuReCentre(const wxCommandEvent& event);

    private:
	wxBitmap*  m_bitmap;
	CLocation  m_centre;
	int        m_resolution;
	EType      m_type;
	bool       m_towns;
	bool       m_sites;
	int        m_textSize;
	wxMenu*    m_menu1;
	wxMenu*    m_menuCoverage;
	wxMenu*    m_menu2;
	wxMenu*    m_menuRes;
	wxMenu*    m_menuType;
	wxMenu*    m_menuTowns;
	wxMenu*    m_menuSites;
	int*       m_map;
	short*     m_colours;
	int*       m_coverage[MAX_COVERAGE_COUNT];
	double     m_ulLat;
	double     m_ulLon;
	double     m_brLat;
	double     m_brLon;
	double     m_stepLat;
	double     m_stepLon;
	int        m_maxHeight;
	int        m_minHeight;
	int        m_mouseX;
	int        m_mouseY;
	int        m_mouseMode;
	int        m_n;
	int        m_dynamicY;
	int        m_height;
	int        m_frequency;
	int        m_polarisation;
	int        m_climate;
	int        m_ground;
	int        m_confidence;
	int        m_txPower;
	int        m_txAntGain;
	int        m_rxPower;
	int        m_rxAntGain;

	DECLARE_EVENT_TABLE()

	void createMapData();
	void drawMap();
	void drawTowns();
	void drawSites();
	void drawFixedData();

	void mouseMapClick(int x, int y);
	void mouseTerrainClick(int x, int y);
	void mouseReCentreClick(int x, int y);

	double xToLongitude(int x) const;
	double yToLatitude(int y) const;
	int    longitudeToX(double lon) const;
	int    latitudeToY(double lat) const;

	int    getNearestPoint(int x1, int y1) const;
	void   refreshMap();

	int    getShade(int n) const;
	void   drawGradient(wxDC& dc, int x, int y, bool r, bool g, bool b);
};

#endif
