/*
 *   Copyright (C) 2002-2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "MapDialog.h"
#include "SiteList.h"
#include "TownList.h"
#include "Defs.h"

enum {
	Resolution_30,
	Resolution_60,
	Resolution_90,
	Resolution_125,
	Resolution_250,
	Resolution_500,
	Resolution_1000,
	Resolution_2000,
	Resolution_4000
};

enum {
	Type_LatLon,
	Type_Locator,
	Type_Name,
	Type_NGR,
	Type_UTM
};

enum {
	Button_Towns = 2628,
	Button_Sites
};

BEGIN_EVENT_TABLE(CMapDialog, wxDialog)
	EVT_CHECKBOX(Button_Towns, CMapDialog::onTowns)
	EVT_CHECKBOX(Button_Sites, CMapDialog::onSites)
	EVT_BUTTON(wxID_OK,        CMapDialog::onOK)
	EVT_BUTTON(wxID_CANCEL,    CMapDialog::onCancel)
END_EVENT_TABLE()

CMapDialog::CMapDialog(wxWindow* parent, EFormat format, int resolution, EType type, bool towns, bool sites, int id) :
wxDialog(parent, id, wxString(wxT("Map Data"))),
m_location(NULL),
m_format(NULL),
m_resolution(NULL),
m_type(NULL),
m_towns(towns),
m_sites(sites)
{
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* boxSizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxVERTICAL);

	wxFlexGridSizer* entrySizer = new wxFlexGridSizer(4);

	wxStaticText* locationLabel = new wxStaticText(this, -1, wxT("Location"));
	entrySizer->Add(locationLabel, 0, wxALL, BORDER_SIZE);

	m_location = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(150, -1));
	m_location->SetMaxLength(25);
	entrySizer->Add(m_location, 0, wxALL, BORDER_SIZE);

	wxStaticText* formatLabel = new wxStaticText(this, -1, wxT("Format"));
	entrySizer->Add(formatLabel, 0, wxALL, BORDER_SIZE);

	m_format = new wxChoice(this, -1, wxDefaultPosition, wxSize(110, -1));
	m_format->Append(wxT("LatLon"),  (void *)Type_LatLon);
	m_format->Append(wxT("Locator"), (void *)Type_Locator);
	m_format->Append(wxT("Name"),    (void *)Type_Name);
	m_format->Append(wxT("NGR"),     (void *)Type_NGR);
//	m_format->Append(wxT("UTM"),     (void *)Type_UTM);
	entrySizer->Add(m_format, 0, wxALL, BORDER_SIZE);

	wxStaticText* townsLabel = new wxStaticText(this, -1, wxT("Display Towns"));
	entrySizer->Add(townsLabel, 0, wxALL, BORDER_SIZE);

	wxCheckBox* townsButton = new wxCheckBox(this, Button_Towns, wxEmptyString);
	entrySizer->Add(townsButton, 0, wxALL, BORDER_SIZE);

	wxStaticText* resLabel = new wxStaticText(this, -1, wxT("Pixel Resolution"));
	entrySizer->Add(resLabel, 0, wxALL, BORDER_SIZE);

	m_resolution = new wxChoice(this, -1, wxDefaultPosition, wxSize(110, -1));
	m_resolution->Append(wxT("30 m"),  (void *)Resolution_30);
	m_resolution->Append(wxT("60 m"),  (void *)Resolution_60);
	m_resolution->Append(wxT("90 m"),  (void *)Resolution_90);
	m_resolution->Append(wxT("125 m"), (void *)Resolution_125);
	m_resolution->Append(wxT("250 m"), (void *)Resolution_250);
	m_resolution->Append(wxT("500 m"), (void *)Resolution_500);
	m_resolution->Append(wxT("1 km"),  (void *)Resolution_1000);
	m_resolution->Append(wxT("2 km"),  (void *)Resolution_2000);
	m_resolution->Append(wxT("4 km"),  (void *)Resolution_4000);
	entrySizer->Add(m_resolution, 0, wxALL, BORDER_SIZE);

	wxStaticText* sitesLabel = new wxStaticText(this, -1, wxT("Display Sites"));
	entrySizer->Add(sitesLabel, 0, wxALL, BORDER_SIZE);

	wxCheckBox* sitesButton = new wxCheckBox(this, Button_Sites, wxEmptyString);
	entrySizer->Add(sitesButton, 0, wxALL, BORDER_SIZE);

	wxStaticText* typeLabel = new wxStaticText(this, -1, wxT("Map Type"));
	entrySizer->Add(typeLabel, 0, wxALL, BORDER_SIZE);

	m_type = new wxChoice(this, -1, wxDefaultPosition, wxSize(110, -1));
	m_type->Append(wxT("Pseudo 3D 1"), (void *)TYPE_PSEUDO_3D_1);
	m_type->Append(wxT("Pseudo 3D 2"), (void *)TYPE_PSEUDO_3D_2);
	m_type->Append(wxT("Elevation"),   (void *)TYPE_ELEVATION);
	entrySizer->Add(m_type, 0, wxALL, BORDER_SIZE);

	boxSizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(boxSizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	switch (format) {
		case FORMAT_LATLON:
			m_format->SetSelection(Type_LatLon);
			break;
		case FORMAT_LOCATOR:
			m_format->SetSelection(Type_Locator);
			break;
		case FORMAT_NGR:
			m_format->SetSelection(Type_NGR);
			break;
//		case FORMAT_UTM:
//			m_format->SetSelection(Type_UTM);
//			break;
		default:
			m_format->SetSelection(Type_Name);
			break;
	}

	switch (resolution) {
		case 30:
			m_resolution->SetSelection(Resolution_30);
			break;
		case 60:
			m_resolution->SetSelection(Resolution_60);
			break;
		case 90:
			m_resolution->SetSelection(Resolution_90);
			break;
		case 125:
			m_resolution->SetSelection(Resolution_125);
			break;
		case 250:
			m_resolution->SetSelection(Resolution_250);
			break;
		case 500:
			m_resolution->SetSelection(Resolution_500);
			break;
		case 1000:
			m_resolution->SetSelection(Resolution_1000);
			break;
		case 2000:
			m_resolution->SetSelection(Resolution_2000);
			break;
		case 4000:
			m_resolution->SetSelection(Resolution_4000);
			break;
		default:
			m_resolution->SetSelection(Resolution_250);
			break;
	}

	m_type->SetSelection(type);

	townsButton->SetValue(m_towns);
	sitesButton->SetValue(m_sites);

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

CMapDialog::~CMapDialog()
{
}

void CMapDialog::onOK(const wxCommandEvent& event)
{
	wxASSERT(m_location != NULL);
	wxASSERT(m_format != NULL);

	wxString text = m_location->GetValue();

	if (text.IsEmpty()) {
		wxMessageDialog dialog(this, wxT("Must enter a Location"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	CLocation* location = getLocation();

	if (location == NULL || !location->isValid()) {
		wxMessageDialog dialog(this, wxT("The Location is not valid"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	if (IsModal()) {
		EndModal(wxID_OK);
	} else {
		SetReturnCode(wxID_OK);
		Show(false);
	}
}

void CMapDialog::onCancel(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_CANCEL);
	} else {
		SetReturnCode(wxID_CANCEL);
		Show(false);
	}
}

void CMapDialog::onTowns(const wxCommandEvent& event)
{
	m_towns = event.IsChecked();
}

void CMapDialog::onSites(const wxCommandEvent& event)
{
	m_sites = event.IsChecked();
}

CLocation* CMapDialog::getLocation() const
{
	wxASSERT(m_location != NULL);
	wxASSERT(m_format != NULL);

	wxString text = m_location->GetValue();
	int format    = m_format->GetSelection();

	CLocation* location = NULL;

	switch (format) {
		case Type_LatLon:
			location = new CLocation(FORMAT_LATLON, text);
			break;
		case Type_Locator:
			location = new CLocation(FORMAT_LOCATOR, text);
			break;
		case Type_Name: {
				CPoint* point = CSiteList::getInstance()->find(text);

				if (point == NULL)
					point = CTownList::getInstance()->find(text);

				if (point != NULL)
					location = new CLocation(point->getLocation());
			}
			break;
		case Type_NGR:
			location = new CLocation(FORMAT_NGR, text);
			break;
//		case Type_UTM:
//			location = new CLocation(FORMAT_UTM, text);
//			break;
	}

	return location;
}

int CMapDialog::getResolution() const
{
	wxASSERT(m_resolution != NULL);

	int resolution = m_resolution->GetSelection();

	switch (resolution) {
		case Resolution_30:
			return 30;
		case Resolution_60:
			return 60;
		case Resolution_90:
			return 90;
		case Resolution_125:
			return 125;
		case Resolution_250:
			return 250;
		case Resolution_500:
			return 500;
		case Resolution_1000:
			return 1000;
		case Resolution_2000:
			return 2000;
		case Resolution_4000:
			return 4000;
		default:
			wxASSERT(false);
			return 0;
	}
}

EType CMapDialog::getType() const
{
	wxASSERT(m_type != NULL);

	return EType(m_type->GetSelection());
}

bool CMapDialog::getTowns() const
{
	return m_towns;
}

bool CMapDialog::getSites() const
{
	return m_sites;
}
