/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "MapFile.h"
#include "Defs.h"

#include <wx/textfile.h>
#include <wx/tokenzr.h>

CMapFile::CMapFile()
{
}

CMapFile::~CMapFile()
{
}

bool CMapFile::read(const wxString& fileName, CLocation& location, int& resolution, EType& type, bool& towns, bool& sites) const
{
	wxTextFile file;

	if (!file.Open(fileName))
		return false;

	int n = file.GetLineCount();

	type = TYPE_PSEUDO_3D_1;

	double latitude = double(VOID_DATA);

	for (int line = 0; line < n; line++) {
		wxString str = file.GetLine(line);

		if (str.Mid(0, 4).IsSameAs(wxT("lat="))) {
			str.Mid(4).ToDouble(&latitude);
		} else if (str.Mid(0, 4).IsSameAs(wxT("lon="))) {
			double longitude;
			str.Mid(4).ToDouble(&longitude);

			location.setLatLon(latitude, longitude);
		} else if (str.Mid(0, 4).IsSameAs(wxT("loc="))) {
			wxString locator = str.Mid(4);

			location.setLocation(FORMAT_LOCATOR, locator);
		} else if (str.Mid(0, 4).IsSameAs(wxT("ngr="))) {
			wxString ngr = str.Mid(4);

			location.setLocation(FORMAT_NGR, ngr);
		} else if (str.Mid(0, 4).IsSameAs(wxT("utm="))) {
			wxString utm = str.Mid(4);

			location.setLocation(FORMAT_UTM, utm);
		} else if (str.Mid(0, 11).IsSameAs(wxT("resolution="))) {
			long res;
			str.Mid(11).ToLong(&res);

			resolution = int(res);
		} else if (str.Mid(0, 5).IsSameAs(wxT("type="))) {
			long typ;
			str.Mid(5).ToLong(&typ);

			type = EType(typ);
		} else if (str.Mid(0, 6).IsSameAs(wxT("towns="))) {
			long twns;
			str.Mid(6).ToLong(&twns);

			towns = (twns == 1L);
		} else if (str.Mid(0, 6).IsSameAs(wxT("sites="))) {
			long sts;
			str.Mid(6).ToLong(&sts);

			sites = (sts == 1L);
		}
	}

	file.Close();

	return true;
}

bool CMapFile::write(const wxString& fileName, const CLocation& location, int resolution, EType type, bool towns, bool sites) const
{
	if (::wxFileExists(fileName)) {
		bool ret = ::wxRemoveFile(fileName);

		if (!ret)
			return false;
	}

	wxTextFile file;

	if (!file.Create(fileName))
		return false;

	wxString text;

	switch (location.getFormat()) {
		case FORMAT_LATLON:
			text.Printf(wxT("lat=%f"), location.getLatitude());
			file.AddLine(text);
			text.Printf(wxT("lon=%f"), location.getLongitude());
			file.AddLine(text);
			break;

		case FORMAT_LOCATOR:
			text.Printf(wxT("loc=%s"), location.getLocation(FORMAT_LOCATOR).c_str());
			file.AddLine(text);
			break;

		case FORMAT_NGR:
			text.Printf(wxT("ngr=%s"), location.getLocation(FORMAT_NGR).c_str());
			file.AddLine(text);
			break;

		case FORMAT_UTM:
			text.Printf(wxT("utm=%s"), location.getLocation(FORMAT_UTM).c_str());
			file.AddLine(text);
			break;
	}

	text.Printf(wxT("resolution=%d"), resolution);
	file.AddLine(text);

	text.Printf(wxT("type=%d"), type);
	file.AddLine(text);

	text.Printf(wxT("towns=%d"), towns ? 1 : 0);
	file.AddLine(text);

	text.Printf(wxT("sites=%d"), sites ? 1 : 0);
	file.AddLine(text);

	file.Write();
	file.Close();

	return true;
}
