/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "MapPreferences.h"
#include "Defs.h"

enum {
	Resolution_30,
	Resolution_60,
	Resolution_90,
	Resolution_125,
	Resolution_250,
	Resolution_500,
	Resolution_1000,
	Resolution_2000,
	Resolution_4000
};

enum {
	Button_Towns = 7251,
	Button_Sites
};

BEGIN_EVENT_TABLE(CMapPreferences, wxDialog)
	EVT_CHECKBOX(Button_Towns, CMapPreferences::onTowns)
	EVT_CHECKBOX(Button_Sites, CMapPreferences::onSites)
	EVT_BUTTON(wxID_OK,        CMapPreferences::onOK)
	EVT_BUTTON(wxID_CANCEL,    CMapPreferences::onCancel)
END_EVENT_TABLE()

CMapPreferences::CMapPreferences(wxWindow* parent, EFormat format, int resolution, EType type, bool towns, bool sites, int textSize, int id) :
wxDialog(parent, id, wxString(wxT("Map Preferences"))),
m_resolution(NULL),
m_type(NULL),
m_format(NULL),
m_towns(towns),
m_sites(sites),
m_textSize(NULL)
{
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* boxSizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxHORIZONTAL);

	wxFlexGridSizer* entrySizer = new wxFlexGridSizer(2);

	wxStaticText* fmtLabel = new wxStaticText(this, -1, wxT("Format"));
	entrySizer->Add(fmtLabel, 0, wxALL, BORDER_SIZE);

	m_format = new wxChoice(this, -1, wxDefaultPosition, wxSize(110, -1));
	m_format->Append(wxT("Lat/Lon"), (void *)FORMAT_LATLON);
	m_format->Append(wxT("Locator"), (void *)FORMAT_LOCATOR);
	m_format->Append(wxT("NGR"),     (void *)FORMAT_NGR);
//	m_format->Append(wxT("UTM"),     (void *)FORMAT_UTM);
	entrySizer->Add(m_format, 0, wxALL, BORDER_SIZE);

	wxStaticText* resLabel = new wxStaticText(this, -1, wxT("Pixel Resolution"));
	entrySizer->Add(resLabel, 0, wxALL, BORDER_SIZE);

	m_resolution = new wxChoice(this, -1, wxDefaultPosition, wxSize(110, -1));
	m_resolution->Append(wxT("30 m"),  (void *)Resolution_30);
	m_resolution->Append(wxT("60 m"),  (void *)Resolution_60);
	m_resolution->Append(wxT("90 m"),  (void *)Resolution_90);
	m_resolution->Append(wxT("125 m"), (void *)Resolution_125);
	m_resolution->Append(wxT("250 m"), (void *)Resolution_250);
	m_resolution->Append(wxT("500 m"), (void *)Resolution_500);
	m_resolution->Append(wxT("1 km"),  (void *)Resolution_1000);
	m_resolution->Append(wxT("2 km"),  (void *)Resolution_2000);
	m_resolution->Append(wxT("4 km"),  (void *)Resolution_4000);
	entrySizer->Add(m_resolution, 0, wxALL, BORDER_SIZE);

	wxStaticText* typeLabel = new wxStaticText(this, -1, wxT("Map Type"));
	entrySizer->Add(typeLabel, 0, wxALL, BORDER_SIZE);

	m_type = new wxChoice(this, -1, wxDefaultPosition, wxSize(110, -1));
	m_type->Append(wxT("Pseudo 3D 1"), (void *)TYPE_PSEUDO_3D_1);
	m_type->Append(wxT("Pseudo 3D 2"), (void *)TYPE_PSEUDO_3D_2);
	m_type->Append(wxT("Elevation"),   (void *)TYPE_ELEVATION);
	entrySizer->Add(m_type, 0, wxALL, BORDER_SIZE);

	wxStaticText* townsLabel = new wxStaticText(this, -1, wxT("Display Towns"));
	entrySizer->Add(townsLabel, 0, wxALL, BORDER_SIZE);

	wxCheckBox* townsBox = new wxCheckBox(this, Button_Towns, wxEmptyString);
	entrySizer->Add(townsBox, 0, wxALL, BORDER_SIZE);

	wxStaticText* sitesLabel = new wxStaticText(this, -1, wxT("Display Sites"));
	entrySizer->Add(sitesLabel, 0, wxALL, BORDER_SIZE);

	wxCheckBox* sitesBox = new wxCheckBox(this, Button_Sites, wxEmptyString);
	entrySizer->Add(sitesBox, 0, wxALL, BORDER_SIZE);

	wxStaticText* textLabel = new wxStaticText(this, -1, wxT("Text Size"));
	entrySizer->Add(textLabel, 0, wxALL, BORDER_SIZE);

	m_textSize = new wxSpinCtrl(this);
	m_textSize->SetRange(4, 20);
	m_textSize->SetValue(textSize);
	entrySizer->Add(m_textSize, 0, wxALL, BORDER_SIZE);

	boxSizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(boxSizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_format->SetSelection(format);

	switch (resolution) {
		case 30:
			m_resolution->SetSelection(Resolution_30);
			break;
		case 60:
			m_resolution->SetSelection(Resolution_60);
			break;
		case 90:
			m_resolution->SetSelection(Resolution_90);
			break;
		case 125:
			m_resolution->SetSelection(Resolution_125);
			break;
		case 250:
			m_resolution->SetSelection(Resolution_250);
			break;
		case 500:
			m_resolution->SetSelection(Resolution_500);
			break;
		case 1000:
			m_resolution->SetSelection(Resolution_1000);
			break;
		case 2000:
			m_resolution->SetSelection(Resolution_2000);
			break;
		case 4000:
			m_resolution->SetSelection(Resolution_4000);
			break;
		default:
			m_resolution->SetSelection(Resolution_250);
			break;
	}

	m_type->SetSelection(type);

	townsBox->SetValue(m_towns);
	sitesBox->SetValue(m_sites);

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

CMapPreferences::~CMapPreferences()
{
}

void CMapPreferences::onOK(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_OK);
	} else {
		SetReturnCode(wxID_OK);
		Show(false);
	}
}

void CMapPreferences::onCancel(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_CANCEL);
	} else {
		SetReturnCode(wxID_CANCEL);
		Show(false);
	}
}

void CMapPreferences::onTowns(const wxCommandEvent& event)
{
	m_towns = event.IsChecked();
}

void CMapPreferences::onSites(const wxCommandEvent& event)
{
	m_sites = event.IsChecked();
}

EFormat CMapPreferences::getFormat() const
{
	wxASSERT(m_format != NULL);

	return EFormat(m_format->GetSelection());
}

int CMapPreferences::getResolution() const
{
	wxASSERT(m_resolution != NULL);

	int resolution = m_resolution->GetSelection();

	switch (resolution) {
		case Resolution_30:
			return 30;
		case Resolution_60:
			return 60;
		case Resolution_90:
			return 90;
		case Resolution_125:
			return 125;
		case Resolution_250:
			return 250;
		case Resolution_500:
			return 500;
		case Resolution_1000:
			return 1000;
		case Resolution_2000:
			return 2000;
		case Resolution_4000:
			return 4000;
		default:
			wxASSERT(false);
			return 0;
	}
}

EType CMapPreferences::getType() const
{
	wxASSERT(m_type != NULL);

	return EType(m_type->GetSelection());
}

bool CMapPreferences::getTowns() const
{
	return m_towns;
}

bool CMapPreferences::getSites() const
{
	return m_sites;
}

int CMapPreferences::getTextSize() const
{
	wxASSERT(m_textSize != NULL);

	return m_textSize->GetValue();
}
