/*
 *   Copyright (C) 2002,2003 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Palette.h"
#include "Defs.h"

const struct {
	int r;
	int g;
	int b;
} colours[] = {
	{255, 255, 255},	// 0, 800m
	{255, 255, 127},	// 0, 600m
	{224, 224, 0  },	// 0, 400m
	{255, 127, 0  },	// 0, 300m
	{255, 63,  0  },	// 0, 250m
	{255, 0,   0  },	// 0, 200m
	{255, 0,   255},	// 0, 150m
	{219, 112, 147},	// 0, 120m
	{0,   255, 0  },	// 0, 90m
	{128, 255, 0  },	// 0, 60m
	{0,   255, 255},	// 0, 40m
	{0,   219, 255},	// 0, 20m
	{0,   0,   255},	// 0, 10m
	{50,  50,  204},	// 0, 1m
	{0,   0,   204},	// 0, 0m

	{255, 255, 255},	// 0, 800m
	{255, 255, 127},	// 0, 600m
	{224, 224, 0  },	// 0, 400m
	{255, 127, 0  },	// 0, 300m
	{255, 0,   0  },	// 0, 250m
	{255, 72,  72 },	// 0, 200m
	{219, 112, 147},	// 0, 150m
	{255, 0,   255},	// 0, 120m
	{204, 127, 50 },	// 0, 90m
	{224, 224, 0  },	// 0, 60m
	{192, 255, 0  },	// 0, 40m
	{0,   255, 0  },	// 0, 20m
	{70,  142, 70 },	// 0, 10m
	{50,  50,  204},	// 0, 1m
	{0,   0,   204},	// 0, 0m

	{255, 255, 255},	// 1, 800m
	{224, 224, 224},	// 1, 600m
	{208, 208, 208},	// 1, 400m
	{192, 192, 192},	// 1, 300m
	{176, 176, 176},	// 1, 250m
	{160, 160, 160},	// 1, 200m
	{144, 144, 144},	// 1, 150m
	{128, 128, 128},	// 1, 120m
	{112, 112, 112},	// 1, 90m
	{96,  96,  96 },	// 1, 60m
	{80,  80,  80 },	// 1, 40m
	{64,  64,  64 },	// 1, 20m
	{48,  48,  48 },	// 1, 10m
	{32,  32,  32 },	// 1, 1m
	{16,  16,  16 },	// 1, 0m

	{255, 255, 255},	// 2, 800m
	{255, 255, 255},	// 2, 600m
	{255, 0,   255},	// 2, 400m
	{255, 0,   255},	// 2, 300m
	{255, 0,   0  },	// 2, 250m
	{255, 0,   0  },	// 2, 200m
	{255, 255, 0  },	// 2, 150m
	{255, 255, 0  },	// 2, 120m
	{0,   255, 0  },	// 2, 90m
	{0,   255, 0  },	// 2, 60m
	{0,   255, 255},	// 2, 40m
	{0,   255, 255},	// 2, 20m
	{0,   0,   255},	// 2, 10m
	{0,   0,   255},	// 2, 1m
	{0,   0,   204}		// 2, 0m

};

const int numPalettes = 8;
const int numColours  = 15;

CPalette* CPalette::s_instance = NULL;

CPalette::CPalette() :
m_colours(NULL),
m_palette(0)
{
	m_colours = new wxColour*[numColours];

	for (int i = 0; i < numColours; i++)
		m_colours[i] = NULL;

	fillinPalette();
}

CPalette::~CPalette()
{
	delete[] m_colours;
}

CPalette* CPalette::getInstance()
{
	if (s_instance == NULL)
		s_instance = new CPalette();

	return s_instance;
}

wxColour& CPalette::getColour(int n) const
{
	wxASSERT(n >= 0 && n < numColours);

	return *m_colours[n];
}

void CPalette::setPaletteNo(int n)
{
	wxASSERT(n >= 0 && n < numPalettes);

	if (n != m_palette) {
		m_palette = n;
		fillinPalette();
	}
}

int CPalette::getPaletteNo() const
{
	return m_palette;
}

int CPalette::getColourCount() const
{
	return numColours;
}

int CPalette::getPaletteCount() const
{
	return numPalettes;
}

void CPalette::nextPalette()
{
	m_palette++;

	if (m_palette == numPalettes)
		m_palette = 0;

	fillinPalette();
}

void CPalette::prevPalette()
{
	m_palette--;

	if (m_palette < 0)
		m_palette = numPalettes - 1;

	fillinPalette();
}

void CPalette::fillinPalette()
{
	int index;

	for (int colour = 0; colour < numColours; colour++) {
		int r, g, b;

		switch (m_palette) {
			case 0:
				r = colour * 17 + 17;
				g = colour * 17 + 17;
				b = colour * 17 + 17;
				break;

			case 1:
				if (colour > 6)
					r = (colour - 7) * 36;
				else
					r = colour * 17 + 17;
				if (colour > 0)
					g = ::min((colour - 1) * 23, 255);
				else
					g = 17;
				if (colour < 5)
					b = colour * 50;
				else if (colour > 10)
					b = (colour - 10) * 63;
				else
					b = ::max((9 - colour) * 40, 0);
				break;

			case 2:
				if (colour > 5)
					r = ::min((colour - 4) * 32, 255);
				else
					r = 0;
				if (colour > 12)
					g = ::min((colour - 12) * 128, 255);
				else if (colour > 4)
					g = ::max((11 - colour) * 24, 0);
				else if (colour > 1)
					g = ::min(colour * 32, 255);
				else
					g = 0;
				if (colour > 10)
					b = ::min((colour - 10) * 60, 255);
				else if (colour > 2)
					b = ::max((6 - colour) * 20, 0);
				else
					b = ::min(colour * 48, 255);
				break;

			case 3:
				if (colour > 6)
					g = ::min((colour - 5) * 32, 255);
				else
					g = 0;
				if (colour > 1)
					r = ::min(colour * 20, 255);
				else
					r = 0;
				if (colour > 10)
					b = ::min((colour - 10) * 60, 255);
				else if (colour > 2)
					b = ::max((6 - colour) * 20, 0);
				else
					b = ::min(colour * 48, 255);
				break;

			case 4:
			case 5:
			case 6:
			case 7:
				index = (m_palette - 4) * numColours + (numColours - colour - 1);
				r = colours[index].r;
				g = colours[index].g;
				b = colours[index].b;
				break;

			default:
				r = g = b = 0;
				wxASSERT(m_palette >= numPalettes);
		}

		if (m_colours[colour] != NULL)
			delete m_colours[colour];

		m_colours[colour] = new wxColour(r, g, b);
	}
}
