/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "PathPreferences.h"

enum {
	Type_SRTM,
	Type_Globe_v1,
	Type_Globe_v2,
	Type_GTOPO30,
	Type_SRTM_Globe_v1,
	Type_SRTM_Globe_v2,
	Type_SRTM_GTOPO30
};

enum {
	Button_Path_1 = 6382,
	Button_Path_2,
	Button_Path_3,
	Button_Path_4,
	Button_Path_5
};

BEGIN_EVENT_TABLE(CPathPreferences, wxDialog)
	EVT_BUTTON(Button_Path_1, CPathPreferences::onBrowse)
	EVT_BUTTON(Button_Path_2, CPathPreferences::onBrowse)
	EVT_BUTTON(Button_Path_3, CPathPreferences::onBrowse)
	EVT_BUTTON(Button_Path_4, CPathPreferences::onBrowse)
	EVT_BUTTON(Button_Path_5, CPathPreferences::onBrowse)
	EVT_BUTTON(wxID_OK,       CPathPreferences::onOK)
	EVT_BUTTON(wxID_CANCEL,   CPathPreferences::onCancel)
END_EVENT_TABLE()

CPathPreferences::CPathPreferences(wxWindow* parent, const wxString& srtmPath, const wxString& globePath, const wxString& gtopo30Path, int dtem, const wxString& dataPath, const wxString& mapQTHPath, int id) :
wxDialog(parent, id, wxString(wxT("Path Preferences"))),
m_srtmPath(NULL),
m_globePath(NULL),
m_gtopo30Path(NULL),
m_dtem(NULL),
m_dataPath(NULL),
m_mapQTHPath(NULL)
{
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* boxSizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxHORIZONTAL);

	wxFlexGridSizer* entrySizer = new wxFlexGridSizer(3);

	wxSize size(300, -1);

	wxStaticText* srtmLabel = new wxStaticText(this, -1, wxT("SRTM File Path"));
	entrySizer->Add(srtmLabel, 0, wxALL, BORDER_SIZE);

	m_srtmPath = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, size);
	m_srtmPath->SetMaxLength(200);
	entrySizer->Add(m_srtmPath, 0, wxALL, BORDER_SIZE);

	wxButton* srtmButton = new wxButton(this, Button_Path_1, wxT("Browse..."));
	entrySizer->Add(srtmButton, 0, wxALL, BORDER_SIZE);

	wxStaticText* globeLabel = new wxStaticText(this, -1, wxT("Globe File Path"));
	entrySizer->Add(globeLabel, 0, wxALL, BORDER_SIZE);

	m_globePath = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, size);
	m_globePath->SetMaxLength(200);
	entrySizer->Add(m_globePath, 0, wxALL, BORDER_SIZE);

	wxButton* globeButton = new wxButton(this, Button_Path_2, wxT("Browse..."));
	entrySizer->Add(globeButton, 0, wxALL, BORDER_SIZE);

	wxStaticText* gtopo30Label = new wxStaticText(this, -1, wxT("GTOPO30 File Path"));
	entrySizer->Add(gtopo30Label, 0, wxALL, BORDER_SIZE);

	m_gtopo30Path = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, size);
	m_gtopo30Path->SetMaxLength(200);
	entrySizer->Add(m_gtopo30Path, 0, wxALL, BORDER_SIZE);

	wxButton* gtopo30Button = new wxButton(this, Button_Path_3, wxT("Browse..."));
	entrySizer->Add(gtopo30Button, 0, wxALL, BORDER_SIZE);

	wxStaticText* dataLabel = new wxStaticText(this, -1, wxT("Data File Path"));
	entrySizer->Add(dataLabel, 0, wxALL, BORDER_SIZE);

	m_dataPath = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, size);
	m_dataPath->SetMaxLength(200);
	entrySizer->Add(m_dataPath, 0, wxALL, BORDER_SIZE);

	wxButton* dataButton = new wxButton(this, Button_Path_4, wxT("Browse..."));
	entrySizer->Add(dataButton, 0, wxALL, BORDER_SIZE);

	wxStaticText* mapQTHLabel = new wxStaticText(this, -1, wxT("Map/QTH File Path"));
	entrySizer->Add(mapQTHLabel, 0, wxALL, BORDER_SIZE);

	m_mapQTHPath = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, size);
	m_mapQTHPath->SetMaxLength(200);
	entrySizer->Add(m_mapQTHPath, 0, wxALL, BORDER_SIZE);

	wxButton* mapQTHButton = new wxButton(this, Button_Path_5, wxT("Browse..."));
	entrySizer->Add(mapQTHButton, 0, wxALL, BORDER_SIZE);

	wxStaticText* dtemLabel = new wxStaticText(this, -1, wxT("DTEM Choice"));
	entrySizer->Add(dtemLabel, 0, wxALL, BORDER_SIZE);

	m_dtem = new wxChoice(this, -1, wxDefaultPosition, wxSize(200, -1));
	m_dtem->Append(wxT("SRTM Only"),           (void *)Type_SRTM);
	m_dtem->Append(wxT("Globe v1.0 Only"),     (void *)Type_Globe_v1);
	m_dtem->Append(wxT("Globe v2.0 Only"),     (void *)Type_Globe_v2);
	m_dtem->Append(wxT("GTOPO30 Only"),        (void *)Type_GTOPO30);
	m_dtem->Append(wxT("SRTM and Globe v1.0"), (void *)Type_SRTM_Globe_v1);
	m_dtem->Append(wxT("SRTM and Globe v2.0"), (void *)Type_SRTM_Globe_v2);
	m_dtem->Append(wxT("SRTM and GTOPO30"),    (void *)Type_SRTM_GTOPO30);
	entrySizer->Add(m_dtem, 0, wxALL, BORDER_SIZE);

	boxSizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(boxSizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	switch (dtem) {
		case DTEM_SRTM:
			m_dtem->SetSelection(Type_SRTM);
			break;
		case DTEM_GLOBE_V1:
			m_dtem->SetSelection(Type_Globe_v1);
			break;
		case DTEM_GLOBE_V2:
			m_dtem->SetSelection(Type_Globe_v2);
			break;
		case DTEM_GTOPO30:
			m_dtem->SetSelection(Type_GTOPO30);
			break;
		case DTEM_SRTM_GLOBE_V1:
			m_dtem->SetSelection(Type_SRTM_Globe_v1);
			break;
		case DTEM_SRTM_GLOBE_V2:
			m_dtem->SetSelection(Type_SRTM_Globe_v2);
			break;
		case DTEM_SRTM_GTOPO30:
			m_dtem->SetSelection(Type_SRTM_GTOPO30);
			break;
	}

	m_srtmPath->SetValue(srtmPath);
	m_globePath->SetValue(globePath);
	m_gtopo30Path->SetValue(gtopo30Path);
	m_dataPath->SetValue(dataPath);
	m_mapQTHPath->SetValue(mapQTHPath);

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

CPathPreferences::~CPathPreferences()
{
}

void CPathPreferences::onBrowse(const wxCommandEvent& event)
{
	wxASSERT(m_srtmPath != NULL);
	wxASSERT(m_globePath != NULL);
	wxASSERT(m_gtopo30Path != NULL);
	wxASSERT(m_dataPath != NULL);
	wxASSERT(m_mapQTHPath != NULL);

	wxString title, path, fileTypes;

	switch (event.GetId()) {
		case Button_Path_1:
			title = wxT("Find SRTM Directory");
			path  = getSRTMPath();
			fileTypes = wxT("SRTM Files (*.hgt.zip;*.HGT.ZIP)|*.hgt.zip;*.HGT.ZIP");
			break;
		case Button_Path_2:
			title = wxT("Find Globe Directory");
			path  = getGlobePath();
			fileTypes = wxT("Globe Files (*g;*G)|*g;*G");
			break;
		case Button_Path_3:
			title = wxT("Find GTOPO30 Directory");
			path  = getGTOPO30Path();
			fileTypes = wxT("GTOPO30 Files (*.hdr;*.dem;*.HDR;*.DEM)|*.hdr;*.dem;*.HDR;*.DEM");
			break;
		case Button_Path_4:
			title = wxT("Find Data Directory");
			path  = getDataPath();
			fileTypes = wxT("Data Files (*.dat;*.DAT)|*.dat;*.DAT");
			break;
		case Button_Path_5:
			title = wxT("Find Map/QTH Directory");
			path  = getMapQTHPath();
			fileTypes = wxT("Data Files (*.map;*.MAP;*.qth;*.QTH)|*.map;*.MAP;*.qth;*.QTH");
			break;
		default:
			wxASSERT(false);
			break;
	}

	wxFileDialog dialog(this, title, path, wxEmptyString, fileTypes);
	dialog.SetStyle(wxOPEN);

	if (dialog.ShowModal() != wxID_OK)
		return;

	path = dialog.GetDirectory();

	switch (event.GetId()) {
		case Button_Path_1:
			m_srtmPath->SetValue(path);
			break;
		case Button_Path_2:
			m_globePath->SetValue(path);
			break;
		case Button_Path_3:
			m_gtopo30Path->SetValue(path);
			break;
		case Button_Path_4:
			m_dataPath->SetValue(path);
			break;
		case Button_Path_5:
			m_mapQTHPath->SetValue(path);
			break;
	}
}

void CPathPreferences::onOK(const wxCommandEvent& event)
{
	wxASSERT(m_srtmPath != NULL);
	wxASSERT(m_globePath != NULL);
	wxASSERT(m_gtopo30Path != NULL);
	wxASSERT(m_dataPath != NULL);
	wxASSERT(m_mapQTHPath != NULL);

	if (IsModal()) {
		EndModal(wxID_OK);
	} else {
		SetReturnCode(wxID_OK);
		Show(false);
	}
}

void CPathPreferences::onCancel(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_CANCEL);
	} else {
		SetReturnCode(wxID_CANCEL);
		Show(false);
	}
}

wxString CPathPreferences::getSRTMPath() const
{
	wxASSERT(m_srtmPath != NULL);

	return m_srtmPath->GetValue();
}

wxString CPathPreferences::getGlobePath() const
{
	wxASSERT(m_globePath != NULL);

	return m_globePath->GetValue();
}

wxString CPathPreferences::getGTOPO30Path() const
{
	wxASSERT(m_gtopo30Path != NULL);

	return m_gtopo30Path->GetValue();
}

EDTEM CPathPreferences::getDTEM() const
{
	wxASSERT(m_dtem != NULL);

	int dtem = m_dtem->GetSelection();

	switch (dtem) {
		case Type_SRTM:
			return DTEM_SRTM;

		case Type_Globe_v1:
			return DTEM_GLOBE_V1;

		case Type_Globe_v2:
			return DTEM_GLOBE_V2;

		case Type_GTOPO30:
			return DTEM_GTOPO30;

		case Type_SRTM_Globe_v1:
			return DTEM_SRTM_GLOBE_V1;

		case Type_SRTM_Globe_v2:
			return DTEM_SRTM_GLOBE_V2;

		case Type_SRTM_GTOPO30:
			return DTEM_SRTM_GTOPO30;

		default:
			wxASSERT(false);
			return DTEM_SRTM_GTOPO30;
	}
}

wxString CPathPreferences::getDataPath() const
{
	wxASSERT(m_dataPath != NULL);

	return m_dataPath->GetValue();
}

wxString CPathPreferences::getMapQTHPath() const
{
	wxASSERT(m_mapQTHPath != NULL);

	return m_mapQTHPath->GetValue();
}
