/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "PointDialog.h"
#include "SiteList.h"
#include "TownList.h"
#include "Defs.h"

enum {
	Type_LatLon,
	Type_Locator,
	Type_Name,
	Type_NGR,
	Type_UTM
};

BEGIN_EVENT_TABLE(CPointDialog, wxDialog)
	EVT_BUTTON(wxID_OK,        CPointDialog::onOK)
	EVT_BUTTON(wxID_CANCEL,    CPointDialog::onCancel)
END_EVENT_TABLE()

CPointDialog::CPointDialog(wxWindow* parent, const CLocation& location, EFormat format, int height, int asl, int id) :
wxDialog(parent, id, wxString(wxT("Point Data"))),
m_name(NULL),
m_location(NULL),
m_format(NULL),
m_height(NULL),
m_asl(NULL)
{
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* boxSizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxHORIZONTAL);

	wxFlexGridSizer* entrySizer = new wxFlexGridSizer(4);

	wxStaticText* locationLabel = new wxStaticText(this, -1, wxT("Location"));
	entrySizer->Add(locationLabel, 0, wxALL, BORDER_SIZE);

	m_location = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(150, -1));
	m_location->SetMaxLength(25);
	entrySizer->Add(m_location, 0, wxALL, BORDER_SIZE);

	wxStaticText* formatLabel = new wxStaticText(this, -1, wxT("Format"));
	entrySizer->Add(formatLabel, 0, wxALL, BORDER_SIZE);

	m_format = new wxChoice(this, -1);
	m_format->Append(wxT("Lat/Lon"), (void *)Type_LatLon);
	m_format->Append(wxT("Locator"), (void *)Type_Locator);
	m_format->Append(wxT("Name"),    (void *)Type_Name);
	m_format->Append(wxT("NGR"),     (void *)Type_NGR);
//	m_format->Append(wxT("UTM"),     (void *)Type_UTM);
	entrySizer->Add(m_format, 0, wxALL, BORDER_SIZE);

	wxStaticText* hgtLabel = new wxStaticText(this, -1, wxT("Height AGL (m)"));
	entrySizer->Add(hgtLabel, 0, wxALL, BORDER_SIZE);

	m_height = new wxTextCtrl(this, -1);
	m_height->SetMaxLength(10);
	entrySizer->Add(m_height, 0, wxALL, BORDER_SIZE);

	wxStaticText* aslLabel = new wxStaticText(this, -1, wxT("Height ASL (m)"));
	entrySizer->Add(aslLabel, 0, wxALL, BORDER_SIZE);

	m_asl = new wxTextCtrl(this, -1);
	m_asl->SetMaxLength(10);
	entrySizer->Add(m_asl, 0, wxALL, BORDER_SIZE);

	wxStaticText* nameLabel = new wxStaticText(this, -1, wxT("Name"));
	entrySizer->Add(nameLabel, 0, wxALL, BORDER_SIZE);

	m_name = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition, wxSize(150, -1));
	m_name->SetMaxLength(25);
	entrySizer->Add(m_name, 0, wxALL, BORDER_SIZE);

	boxSizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(boxSizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	if (height != VOID_DATA) {
		wxString text;
		text.Printf(wxT("%d"), height);

		m_height->SetValue(text);
	}

	if (asl != VOID_DATA) {
		wxString text;
		text.Printf(wxT("%d"), asl);

		m_asl->SetValue(text);
	}

	switch (format) {
		case FORMAT_LATLON:
			m_format->SetSelection(Type_LatLon);
			break;
		case FORMAT_LOCATOR:
			m_format->SetSelection(Type_Locator);
			break;
		case FORMAT_NGR:
			m_format->SetSelection(Type_NGR);
			break;
//		case FORMAT_UTM:
//			m_format->SetSelection(Type_UTM);
//			break;
		default:
			m_format->SetSelection(Type_Name);
			break;
	}

	m_location->SetValue(location.getLocation(format));

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

CPointDialog::~CPointDialog()
{
}

void CPointDialog::onOK(const wxCommandEvent& event)
{
	wxASSERT(m_name != NULL);
	wxASSERT(m_location != NULL);
	wxASSERT(m_format != NULL);
	wxASSERT(m_height != NULL);
	wxASSERT(m_asl != NULL);

	wxString text = m_location->GetValue();

	if (text.IsEmpty()) {
		wxMessageDialog dialog(this, wxT("Must enter a Location"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	CLocation* location = getLocation();

	if (location == NULL || !location->isValid()) {
		wxMessageDialog dialog(this, wxT("The Location is not valid"), wxT("UKW Tools Error"), wxICON_ERROR);
		dialog.ShowModal();
		return;
	}

	if (IsModal()) {
		EndModal(wxID_OK);
	} else {
		SetReturnCode(wxID_OK);
		Show(false);
	}
}

void CPointDialog::onCancel(const wxCommandEvent& event)
{
	if (IsModal()) {
		EndModal(wxID_CANCEL);
	} else {
		SetReturnCode(wxID_CANCEL);
		Show(false);
	}
}

wxString CPointDialog::getName() const
{
	wxASSERT(m_name != NULL);

	return m_name->GetValue();
}

CLocation* CPointDialog::getLocation() const
{
	wxASSERT(m_location != NULL);
	wxASSERT(m_format != NULL);

	wxString text = m_location->GetValue();
	int format    = m_format->GetSelection();

	CLocation* location = NULL;

	switch (format) {
		case Type_LatLon:
			location = new CLocation(FORMAT_LATLON, text);
			break;
		case Type_Locator:
			location = new CLocation(FORMAT_LOCATOR, text);
			break;
		case Type_Name: {
				CPoint* point = CSiteList::getInstance()->find(text);

				if (point == NULL)
					point = CTownList::getInstance()->find(text);

				if (point != NULL)
					location = new CLocation(point->getLocation());
			}
			break;
		case Type_NGR:
			location = new CLocation(FORMAT_NGR, text);
			break;
//		case Type_UTM:
//			location = new CLocation(FORMAT_UTM, text);
//			break;
	}

	return location;
}

int CPointDialog::getHeight() const
{
	wxASSERT(m_height != NULL);

	long height;
	m_height->GetValue().ToLong(&height);

	return int(height);
}

int CPointDialog::getASL() const
{
	wxASSERT(m_asl != NULL);

	wxString text = m_asl->GetValue();

	if (text.IsEmpty())
		return VOID_DATA;

	long asl;
	text.ToLong(&asl);

	return int(asl);
}
