/*
 *   Copyright (C) 2003,2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "PointFile.h"
#include "Defs.h"

#include <wx/textfile.h>
#include <wx/tokenzr.h>

CPointFile::CPointFile()
{
}

CPointFile::~CPointFile()
{
}

bool CPointFile::read(const wxString& fileName, wxString& name, CLocation& location, int& height, int& asl) const
{
	wxTextFile file;

	if (!file.Open(fileName))
		return false;

	int n = file.GetLineCount();

	double latitude = double(VOID_DATA);

	name = wxEmptyString;
	asl  = VOID_DATA;

	for (int line = 0; line < n; line++) {
		wxString str = file.GetLine(line);

		if (str.Mid(0, 5).IsSameAs(wxT("name="))) {
			name = str.Mid(5);
		} else if (str.Mid(0, 4).IsSameAs(wxT("lat="))) {
			str.Mid(4).ToDouble(&latitude);
		} else if (str.Mid(0, 4).IsSameAs(wxT("lon="))) {
			double longitude;
			str.Mid(4).ToDouble(&longitude);

			location.setLatLon(latitude, longitude);
		} else if (str.Mid(0, 4).IsSameAs(wxT("loc="))) {
			wxString locator = str.Mid(4);

			location.setLocation(FORMAT_LOCATOR, locator);
		} else if (str.Mid(0, 4).IsSameAs(wxT("ngr="))) {
			wxString ngr = str.Mid(4);

			location.setLocation(FORMAT_NGR, ngr);
		} else if (str.Mid(0, 4).IsSameAs(wxT("utm="))) {
			wxString utm = str.Mid(4);

			location.setLocation(FORMAT_UTM, utm);
		} else if (str.Mid(0, 7).IsSameAs(wxT("height="))) {
			long hgt;
			str.Mid(7).ToLong(&hgt);

			height = int(hgt);
		} else if (str.Mid(0, 4).IsSameAs(wxT("asl="))) {
			long hgt;
			str.Mid(4).ToLong(&hgt);

			asl = int(hgt);
		}
	}

	file.Close();

	return true;
}

bool CPointFile::write(const wxString& fileName, const wxString& name, const CLocation& location, int height, int asl) const
{
	if (::wxFileExists(fileName)) {
		bool ret = ::wxRemoveFile(fileName);

		if (!ret)
			return false;
	}

	wxTextFile file;

	if (!file.Create(fileName))
		return false;

	wxString text;

	text.Printf(wxT("name=%s"), name.c_str());
	file.AddLine(text);

	switch (location.getFormat()) {
		case FORMAT_LATLON:
			text.Printf(wxT("lat=%f"), location.getLatitude());
			file.AddLine(text);
			text.Printf(wxT("lon=%f"), location.getLongitude());
			file.AddLine(text);
			break;

		case FORMAT_LOCATOR:
			text.Printf(wxT("loc=%s"), location.getLocation(FORMAT_LOCATOR).c_str());
			file.AddLine(text);
			break;

		case FORMAT_NGR:
			text.Printf(wxT("ngr=%s"), location.getLocation(FORMAT_NGR).c_str());
			file.AddLine(text);
			break;

		case FORMAT_UTM:
			text.Printf(wxT("utm=%s"), location.getLocation(FORMAT_UTM).c_str());
			file.AddLine(text);
			break;
	}

	text.Printf(wxT("height=%d"), height);
	file.AddLine(text);

	if (asl != VOID_DATA) {
		text.Printf(wxT("asl=%d"), asl);
		file.AddLine(text);
	}

	file.Write();
	file.Close();

	return true;
}
