/*
 *   Copyright (C) 2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "PointProperties.h"
#include "NGR.h"
#include "App.h"

CPointProperties::CPointProperties(wxWindow* parent, const CPoint& point, int id) :
wxDialog(parent, id, wxString(wxT("Point Properties")))
{
	CDEMCache* cache = ::wxGetApp().getDEMCache();

	wxASSERT(cache != NULL);

	CLocation location = point.getLocation();

	double latitude, longitude;
	location.getLatLon(latitude, longitude);

	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);

	wxStaticBoxSizer* boxSizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxEmptyString), wxHORIZONTAL);

	wxFlexGridSizer* entrySizer = new wxFlexGridSizer(2);

	wxStaticText* nameLabel = new wxStaticText(this, -1, wxT("Point Name:"));
	entrySizer->Add(nameLabel, 0, wxALL, BORDER_SIZE);

	wxString text;
	if (point.getName().IsEmpty())
		text = wxT("<none>");
	else
		text = point.getName();

	wxStaticText* nameText = new wxStaticText(this, -1, text);
	entrySizer->Add(nameText, 0, wxALL, BORDER_SIZE);

	wxStaticText* fileLabel = new wxStaticText(this, -1, wxT("File Name:"));
	entrySizer->Add(fileLabel, 0, wxALL, BORDER_SIZE);

	if (point.getFileName().IsEmpty())
		text = wxT("<none>");
	else
		text = point.getFileName();

	wxStaticText* fileText = new wxStaticText(this, -1, text);
	entrySizer->Add(fileText, 0, wxALL, BORDER_SIZE);

	wxStaticText* latlonLabel = new wxStaticText(this, -1, wxT("Latitude/Longitude:"));
	entrySizer->Add(latlonLabel, 0, wxALL, BORDER_SIZE);

	wxStaticText* latlonText = new wxStaticText(this, -1, location.getLocation(FORMAT_LATLON));
	entrySizer->Add(latlonText, 0, wxALL, BORDER_SIZE);

	wxStaticText* locLabel = new wxStaticText(this, -1, wxT("Locator:"));
	entrySizer->Add(locLabel, 0, wxALL, BORDER_SIZE);

	wxStaticText* locText = new wxStaticText(this, -1, location.getLocation(FORMAT_LOCATOR));
	entrySizer->Add(locText, 0, wxALL, BORDER_SIZE);

	if (location.isValid(FORMAT_NGR)) {
		wxStaticText* ngrLabel = new wxStaticText(this, -1, wxT("NGR:"));
		entrySizer->Add(ngrLabel, 0, wxALL, BORDER_SIZE);

		wxStaticText* ngrText = new wxStaticText(this, -1, location.getLocation(FORMAT_NGR));
		entrySizer->Add(ngrText, 0, wxALL, BORDER_SIZE);

		CNGR ngr;

		int* sheets = ngr.findSheets(location.getLocation(FORMAT_NGR));

		for (int i = 0; sheets[i] != 0; i++) {
			wxStaticText* sheetLabel = new wxStaticText(this, -1, wxT("OS 1\" Map:"));
			entrySizer->Add(sheetLabel, 0, wxALL, BORDER_SIZE);

			wxString name = ngr.findMap(sheets[i]);
			text.Printf(wxT("%d - %s"), sheets[i], name.c_str());

			wxStaticText* sheetText = new wxStaticText(this, -1, text);
			entrySizer->Add(sheetText, 0, wxALL, BORDER_SIZE);
		}

		delete[] sheets;
	}

	wxStaticText* heightLabel = new wxStaticText(this, -1, wxT("Height:"));
	entrySizer->Add(heightLabel, 0, wxALL, BORDER_SIZE);

	int height = 0;
	if (point.getASL() != VOID_DATA)
		height = point.getASL();
	else
		height = cache->getHeight(latitude, longitude);
	text.Printf(wxT("%dm ASL"), height);

	wxStaticText* heightText = new wxStaticText(this, -1, text);
	entrySizer->Add(heightText, 0, wxALL, BORDER_SIZE);

	wxStaticText* mastLabel = new wxStaticText(this, -1, wxT("Mast:"));
	entrySizer->Add(mastLabel, 0, wxALL, BORDER_SIZE);

	text.Printf(wxT("%dm AGL"), point.getHeight());

	wxStaticText* mastText = new wxStaticText(this, -1, text);
	entrySizer->Add(mastText, 0, wxALL, BORDER_SIZE);

	wxStaticText* coverLabel = new wxStaticText(this, -1, wxT("Coverage Plot:"));
	entrySizer->Add(coverLabel, 0, wxALL, BORDER_SIZE);

	text = (point.getCoverage()) ? wxT("Yes") : wxT("No");

	wxStaticText* coverText = new wxStaticText(this, -1, text);
	entrySizer->Add(coverText, 0, wxALL, BORDER_SIZE);

	boxSizer->Add(entrySizer, 0, wxALL, BORDER_SIZE);

	sizer->Add(boxSizer, 0, wxALL, BORDER_SIZE);

	SetAutoLayout(true);
	Layout();

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}

CPointProperties::~CPointProperties()
{
}
