/*
 *   Copyright (C) 2003 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "SRTMCache.h"
#include "Defs.h"

#include <wx/log.h>

const int CACHE_SIZE = 20;

CSRTMCache::CSRTMCache(const wxString& srtmPath) :
m_path(srtmPath),
m_list(NULL),
m_ptr(0),
m_cache(NULL)
{
	m_list = new CSRTM*[CACHE_SIZE];

	for (int i = 0; i < CACHE_SIZE; i++)
		m_list[i] = NULL;
}

CSRTMCache::CSRTMCache(const wxString& srtmPath, CDEMCache* cache) :
m_path(srtmPath),
m_list(NULL),
m_ptr(0),
m_cache(cache)
{
	m_list = new CSRTM*[CACHE_SIZE];

	for (int i = 0; i < CACHE_SIZE; i++)
		m_list[i] = NULL;
}

CSRTMCache::~CSRTMCache()
{
	for (int i = 0; i < CACHE_SIZE; i++)
		if (m_list[i] != NULL)
			delete m_list[i];

	delete[] m_list;

	if (m_cache != NULL)
		delete m_cache;
}

int CSRTMCache::getHeight(double latitude, double longitude, bool interpolate)
{
	if (longitude > 180.0)
		longitude -= 360.0;

	if (longitude < -180.0)
		longitude += 360.0;

	for (int i = 0; i < CACHE_SIZE; i++) {
		if (m_list[i] != NULL) {
			if (m_list[i]->isInData(latitude, longitude)) {
				int height = m_list[i]->getHeight(latitude, longitude);

				int newHeight = checkHeight(height, latitude, longitude);

				if (newHeight != height) {
					m_list[i]->setHeight(newHeight, latitude, longitude);
					height = newHeight;
				}

				return height;
			}
		}
	}

	if (m_list[m_ptr] != NULL)
		delete m_list[m_ptr];

	m_list[m_ptr] = new CSRTM(m_path, latitude, longitude);

	int height = m_list[m_ptr]->getHeight(latitude, longitude);

	int newHeight = checkHeight(height, latitude, longitude);

	if (newHeight != height) {
		m_list[m_ptr]->setHeight(newHeight, latitude, longitude);
		height = newHeight;
	}

	m_ptr++;

	if (m_ptr == CACHE_SIZE)
		m_ptr = 0;

	return height;
}

bool CSRTMCache::dataExists(double latitude, double longitude)
{
	for (int i = 0; i < CACHE_SIZE; i++) {
		if (m_list[i] != NULL) {
			if (m_list[i]->isInData(latitude, longitude))
				return m_list[i]->dataExists();
		}
	}

	if (m_list[m_ptr] != NULL)
		delete m_list[m_ptr];

	m_list[m_ptr] = new CSRTM(m_path, latitude, longitude);

	bool exists = m_list[m_ptr]->dataExists();

	m_ptr++;

	if (m_ptr == CACHE_SIZE)
		m_ptr = 0;

	return exists;
}

int CSRTMCache::checkHeight(int height, double latitude, double longitude) const
{
	if (m_cache == NULL)
		return height;

	if (height < -10 || height == VOID_DATA) {
		if (!m_cache->dataExists(latitude, longitude))
			return VOID_DATA;

		return m_cache->getHeight(latitude, longitude, true);
	} else if (height < 10) {
		if (!m_cache->dataExists(latitude, longitude))
			return height;

		int nheight = m_cache->getHeight(latitude, longitude, true);

		return (nheight == VOID_DATA) ? VOID_DATA : height;
	}

	return height;
}
