/*
 *   Copyright (C) 2002-2004 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef	Terrain_H
#define	Terrain_H

#include <wx/wx.h>

#include "DEMCache.h"
#include "Point.h"
#include "Location.h"
#include "Path.h"

class CTerrain : public wxPanel {

    public:
	CTerrain(wxWindow* parent, const CPoint& start, const CLocation& end, int endHeight, int endASL, int frequency, int polarisation, int climate, int ground, int confidence, int txPower, int txAntGain, int rxPower, int rxAntGain);
	virtual ~CTerrain();

	virtual void onPaint(const wxPaintEvent& event);
	virtual void onMouseMove(const wxMouseEvent& event);

    private:
	wxBitmap*  m_bitmap;
	CPath      m_path;
	CPoint     m_start;
	CLocation  m_end;
	int*       m_groundHeights;
	int*       m_spotHeights;
	double*    m_distances;
	int        m_maxHeight;
	int        m_count;
	int        m_startHeight;
	int        m_endHeight;
	int        m_txHeight;
	int        m_rxHeight;
	int        m_frequency;
	int        m_polarisation;
	int        m_txPower;
	int        m_txAntGain;
	int        m_rxPower;
	int        m_rxAntGain;
	int        m_climate;
	int        m_ground;
	int        m_confidence;

	DECLARE_EVENT_TABLE()

	void drawTerrain();

	void drawLine(wxDC& dc, int x1, int y1, int x2, int y2) const;
	void printTheta(wxDC& dc, int x, int y, bool line);

	int xToCount(int x) const;
	int yToHeight(int y) const;
	int countToX(int count) const;
	int heightToY(int height) const;

	int getFresnel(int n, double d1, double distance) const;
};

#endif
